@@ -16,19 +16,19 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+   02110-1301, USA.
 */
 
 #include <config.h>
 
 #include "edit.h"
-#include "src/wtools.h"		/* query_dialog() */
-#include "src/menu.h"		/* menu_entry */
-#include "src/tty.h"		/* KEY_F */
-#include "src/key.h"		/* XCTRL */
-#include "src/main.h"		/* drop_menus */
-#include "src/learn.h"		/* learn_keys */
+#include "../src/wtools.h"	/* query_dialog() */
+#include "../src/menu.h"	/* menu_entry */
+#include "../src/tty.h"		/* KEY_F */
+#include "../src/key.h"		/* XCTRL */
+#include "../src/main.h"	/* drop_menus */
+#include "../src/learn.h"	/* learn_keys */
 
 #include "edit-widget.h"
 #include "editcmddef.h"
@@ -37,9 +37,11 @@
 #define edit_message_dialog(w,x,y,h,s) query_dialog (h, s, 0, 1, _("&OK"))
 #define CFocus(x) 
 
-static void menu_cmd (int i)
+static void
+menu_cmd (int command)
 {
-    send_message ((Widget *) wedit, WIDGET_COMMAND, i);
+    edit_execute_key_command (wedit, command, -1);
+    edit_update_screen (wedit);
 }
 
 static void menu_key (int i)
@@ -205,6 +207,12 @@ menu_sort_cmd (void)
     menu_cmd (CK_Sort);
 }
 
+static void 
+menu_ext_cmd (void)
+{
+    menu_cmd (CK_ExtCmd);
+}
+
 static void
 menu_date_cmd (void)
 {
@@ -323,22 +331,7 @@ static menu_entry EditMenu[] =
     {' ', N_("&End           C-PgDn"), 'E', menu_end_cmd}
 };
 
-static menu_entry EditMenuEmacs[] =
-{
-    {' ', N_("&Toggle Mark       F3"), 'T', menu_mark_cmd},
-    {' ', N_("&Mark Columns    S-F3"), 'T', menu_markcol_cmd},
-    {' ', "", ' ', 0},
-    {' ', N_("Toggle &ins/overw Ins"), 'I', menu_ins_cmd},
-    {' ', "", ' ', 0},
-    {' ', N_("&Copy              F5"), 'C', menu_copy_cmd},
-    {' ', N_("&Move              F6"), 'M', menu_move_cmd},
-    {' ', N_("&Delete            F8"), 'D', menu_delete_cmd},
-    {' ', "", ' ', 0},
-    {' ', N_("&Undo             C-u"), 'U', menu_undo_cmd},
-    {' ', "", ' ', 0},
-    {' ', N_("&Beginning     C-PgUp"), 'B', menu_beginning_cmd},
-    {' ', N_("&End           C-PgDn"), 'E', menu_end_cmd}
-};
+#define EditMenuEmacs EditMenu
 
 static menu_entry SearReplMenu[] =
 {
@@ -347,12 +340,7 @@ static menu_entry SearReplMenu[] =
     {' ', N_("&Replace...        F4"), 'R', menu_replace_cmd}
 };
 
-static menu_entry SearReplMenuEmacs[] =
-{
-    {' ', N_("&Search...         F7"), 'S', menu_search_cmd},
-    {' ', N_("Search &again     F17"), 'A', menu_search_again_cmd},
-    {' ', N_("&Replace...        F4"), 'R', menu_replace_cmd}
-};
+#define SearReplMenuEmacs SearReplMenu
 
 static menu_entry CmdMenu[] =
 {
@@ -373,6 +361,7 @@ static menu_entry CmdMenu[] =
     {' ', N_("Format p&aragraph        M-p"), 'A', menu_format_paragraph},
     {' ', N_("'ispell' s&pell check    C-p"), 'P', menu_ispell_cmd},
     {' ', N_("Sor&t...                 M-t"), 'T', menu_sort_cmd},
+    {' ', N_("Paste o&utput of...      M-u"), 'U', menu_ext_cmd},
     {' ', N_("E&xternal Formatter      F19"), 'C', menu_c_form_cmd},
     {' ', N_("&Mail...                    "), 'M', menu_mail_cmd}
 };
@@ -396,6 +385,7 @@ static menu_entry CmdMenuEmacs[] =
     {' ', N_("Format p&aragraph        M-p"), 'a', menu_format_paragraph},
     {' ', N_("'ispell' s&pell check    M-$"), 'P', menu_ispell_cmd},
     {' ', N_("Sor&t...                 M-t"), 'T', menu_sort_cmd},
+    {' ', N_("Paste o&utput of...      M-u"), 'U', menu_ext_cmd},
     {' ', N_("E&xternal Formatter      F19"), 'C', menu_c_form_cmd},
     {' ', N_("&Mail...                    "), 'M', menu_mail_cmd}
 };
@@ -404,19 +394,10 @@ static menu_entry OptMenu[] =
 {
     {' ', N_("&General...  "), 'G', menu_options},
     {' ', N_("&Save mode..."), 'S', menu_save_mode_cmd},
-#ifndef NATIVE_WIN32
     {' ', N_("learn &Keys..."), 'K', learn_keys}
-#endif				/* !NATIVE_WIN32 */
 };
 
-static menu_entry OptMenuEmacs[] =
-{
-    {' ', N_("&General...  "), 'G', menu_options},
-    {' ', N_("&Save mode..."), 'S', menu_save_mode_cmd},
-#ifndef NATIVE_WIN32
-    {' ', N_("learn &Keys..."), 'K', learn_keys}
-#endif				/* !NATIVE_WIN32 */
-};
+#define OptMenuEmacs OptMenu
 
 #define menu_entries(x) sizeof(x)/sizeof(menu_entry)
 
@@ -459,7 +440,7 @@ void edit_done_menu (void)
 
 
 static void
-edit_drop_menu_cmd (WEdit * e, int which)
+edit_drop_menu_cmd (WEdit *e, int which)
 {
     if (edit_menubar->active)
 	return;
@@ -467,12 +448,9 @@ edit_drop_menu_cmd (WEdit * e, int which)
     edit_menubar->dropped = drop_menus;
     if (which >= 0) {
 	edit_menubar->selected = which;
-	edit_menubar->previous_selection = which;
-    } else {
-	edit_menubar->previous_selection =
-	    dlg_item_number (e->widget.parent);
     }
 
+    edit_menubar->previous_widget = e->widget.parent->current->dlg_id;
     dlg_select_widget (e->widget.parent, edit_menubar);
 }
 
