@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #include <config.h>
@@ -22,32 +22,22 @@
 #include <errno.h>	/* For errno on SunOS systems	      */
 /* Needed for the extern declarations of integer parameters */
 #include <sys/types.h>
-#include <sys/param.h>
 #include <sys/stat.h>
-#include <grp.h>
-#include <pwd.h>
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #endif
-#include "tty.h"
-#include "global.h"
-#include "win.h"
-#include "color.h"
-#include "dlg.h"
-#include "widget.h"
-#include "dialog.h"	/* For do_refresh() */
 
-#include "dir.h"
-#include "panel.h"		/* Needed for the externs */
-#include "file.h"
-#include "main.h"
+#include "global.h"
+#include "tty.h"	/* A_REVERSE */
+#include "color.h"	/* dialog_colors */
+#include "dialog.h"	/* add_widget() */
+#include "widget.h"	/* NORMAL_BUTTON */
+#include "wtools.h"	/* message() */
+#include "panel.h"	/* do_file_mark() */
+#include "main.h"	/* update_panels() */
 #include "chmod.h"
-#include "achown.h"
-#include "chown.h"
-#include "../vfs/vfs.h"
 
 static int single_set;
-static struct Dlg_head *ch_dlg;
 
 #define PX		5
 #define PY		2
@@ -65,24 +55,26 @@ static struct Dlg_head *ch_dlg;
 #define BUTTONS		6
 
 #define B_MARKED	B_USER
-#define B_ALL		B_USER+1
-#define B_SETMRK        B_USER+2
-#define B_CLRMRK        B_USER+3
+#define B_ALL		(B_USER+1)
+#define B_SETMRK        (B_USER+2)
+#define B_CLRMRK        (B_USER+3)
 
 static int mode_change, need_update;
 static int c_file, end_chmod;
 
 static umode_t and_mask, or_mask, c_stat;
-static char *c_fname, *c_fown, *c_fgrp;
+
+/* FIXME: these variables are superfluous, aren't they? (hint: name_trunc
+ * returns a pointer to a static buffer, and label_new creates its own copy
+ * of its argument)
+ * --rillig, 2004-08-29 */
+static const char *c_fname, *c_fown, *c_fgrp;
 
 static WLabel *statl;
-static int normal_color;
-static int title_color;
-static int selection_color;
 
 static struct {
     mode_t mode;
-    char *text;
+    const char *text;
     int selected;
     WCheck *check;
 } check_perm[PERMISSIONS] = 
@@ -103,7 +95,7 @@ static struct {
 
 static struct {
     int ret_cmd, flags, y, x;
-    char *text;
+    const char *text;
 } chmod_but[BUTTONS] = 
 {
     { B_CANCEL, NORMAL_BUTTON,  2, 33, N_("&Cancel") },
@@ -114,136 +106,121 @@ static struct {
     { B_ALL,    NORMAL_BUTTON,  0, 0,  N_("Set &all") },
 };
 
-static void chmod_toggle_select (void)
-{
-    int Id = ch_dlg->current->dlg_id - BUTTONS + single_set * 2;
-
-    attrset (normal_color);
-    check_perm[Id].selected ^= 1;
-
-    dlg_move (ch_dlg, PY + PERMISSIONS - Id, PX + 1);
-    addch ((check_perm[Id].selected) ? '*' : ' ');
-    dlg_move (ch_dlg, PY + PERMISSIONS - Id, PX + 3);
-}
-
-static void chmod_refresh (void)
+static void chmod_toggle_select (Dlg_head *h, int Id)
 {
     attrset (COLOR_NORMAL);
-    dlg_erase (ch_dlg);
-    
-    draw_box (ch_dlg, 1, 2, 20 - single_set, 66);
-    draw_box (ch_dlg, PY, PX, PERMISSIONS + 2, 33);
-    draw_box (ch_dlg, FY, FX, 10, 25);
+    check_perm[Id].selected ^= 1;
 
-    dlg_move (ch_dlg, FY + 1, FX + 2);
+    dlg_move (h, PY + PERMISSIONS - Id, PX + 1);
+    addch ((check_perm[Id].selected) ? '*' : ' ');
+    dlg_move (h, PY + PERMISSIONS - Id, PX + 3);
+}
+
+static void chmod_refresh (Dlg_head *h)
+{
+    common_dialog_repaint (h);
+
+    attrset (COLOR_NORMAL);
+    
+    draw_box (h, PY, PX, PERMISSIONS + 2, 33);
+    draw_box (h, FY, FX, 10, 25);
+
+    dlg_move (h, FY + 1, FX + 2);
     addstr (_("Name"));
-    dlg_move (ch_dlg, FY + 3, FX + 2);
+    dlg_move (h, FY + 3, FX + 2);
     addstr (_("Permissions (Octal)"));
-    dlg_move (ch_dlg, FY + 5, FX + 2);
+    dlg_move (h, FY + 5, FX + 2);
     addstr (_("Owner name"));
-    dlg_move (ch_dlg, FY + 7, FX + 2);
+    dlg_move (h, FY + 7, FX + 2);
     addstr (_("Group name"));
     
-    attrset (title_color);
-    dlg_move (ch_dlg, 1, 28);
-    addstr (_(" Chmod command "));
-    dlg_move (ch_dlg, PY, PX + 1);
-    addstr (_(" Permission "));
-    dlg_move (ch_dlg, FY, FX + 1);
-    addstr (_(" File "));
-    
-    attrset (selection_color);
-
-    dlg_move (ch_dlg, TY, TX);
+    dlg_move (h, TY, TX);
     addstr (_("Use SPACE to change"));
-    dlg_move (ch_dlg, TY + 1, TX);
+    dlg_move (h, TY + 1, TX);
     addstr (_("an option, ARROW KEYS"));
-    dlg_move (ch_dlg, TY + 2, TX);
+    dlg_move (h, TY + 2, TX);
     addstr (_("to move between options"));
-    dlg_move (ch_dlg, TY + 3, TX);
+    dlg_move (h, TY + 3, TX);
     addstr (_("and T or INS to mark"));
+
+    attrset (COLOR_HOT_NORMAL);
+
+    dlg_move (h, PY, PX + 1);
+    addstr (_(" Permission "));
+    dlg_move (h, FY, FX + 1);
+    addstr (_(" File "));
 }
 
-static int chmod_callback (Dlg_head *h, int Par, int Msg)
+static cb_ret_t
+chmod_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    char buffer [BUF_TINY];
-    
-    switch (Msg) {
+    char buffer[BUF_TINY];
+    int id = h->current->dlg_id - BUTTONS + single_set * 2;
+
+    switch (msg) {
     case DLG_ACTION:
-	if (Par >= BUTTONS - single_set * 2){
-	    c_stat ^= check_perm[Par - BUTTONS + single_set * 2].mode;
+	if (id >= 0) {
+	    c_stat ^= check_perm[id].mode;
 	    g_snprintf (buffer, sizeof (buffer), "%o", c_stat);
 	    label_set_text (statl, buffer);
-	    chmod_toggle_select ();
+	    chmod_toggle_select (h, id);
 	    mode_change = 1;
 	}
-	break;
+	return MSG_HANDLED;
 
     case DLG_KEY:
-	if ((Par == 'T' || Par == 't' || Par == KEY_IC) &&
-	    ch_dlg->current->dlg_id >= BUTTONS - single_set * 2) {
-	    chmod_toggle_select ();
-	    if (Par == KEY_IC)
-		dlg_one_down (ch_dlg);
-	    return 1;
+	if ((parm == 'T' || parm == 't' || parm == KEY_IC) && id > 0) {
+	    chmod_toggle_select (h, id);
+	    if (parm == KEY_IC)
+		dlg_one_down (h);
+	    return MSG_HANDLED;
 	}
-	break;
+	return MSG_NOT_HANDLED;
 
     case DLG_DRAW:
-	chmod_refresh ();
-	break;
+	chmod_refresh (h);
+	return MSG_HANDLED;
+
+    default:
+	return default_dlg_callback (h, msg, parm);
     }
-    return 0;
 }
 
-static void init_chmod (void)
+static Dlg_head *
+init_chmod (void)
 {
     int i;
+    Dlg_head *ch_dlg;
 
     do_refresh ();
     end_chmod = c_file = need_update = 0;
-    single_set = (cpanel->marked < 2) ? 2 : 0;
+    single_set = (current_panel->marked < 2) ? 2 : 0;
 
-    if (use_colors){
-	normal_color = COLOR_NORMAL;
-	title_color  = COLOR_HOT_NORMAL;
-	selection_color = COLOR_NORMAL;
-    } else {
-	normal_color = NORMAL_COLOR;
-	title_color  = SELECTED_COLOR;
-	selection_color = SELECTED_COLOR;
-    }
-    
-    ch_dlg = create_dlg (0, 0, 22 - single_set, 70, dialog_colors,
-			 chmod_callback, "[Chmod]", "chmod", DLG_CENTER);
-			 
-    x_set_dialog_title (ch_dlg, _("Chmod command"));
-
-#define XTRACT(i) BY+chmod_but[i].y-single_set, BX+chmod_but[i].x, \
-     chmod_but[i].ret_cmd, chmod_but[i].flags, _(chmod_but[i].text), 0, 0, NULL
+    ch_dlg =
+	create_dlg (0, 0, 22 - single_set, 70, dialog_colors,
+		    chmod_callback, "[Chmod]", _("Chmod command"),
+		    DLG_CENTER | DLG_REVERSE);
 
     for (i = 0; i < BUTTONS; i++) {
 	if (i == 2 && single_set)
 	    break;
 	else
-	    add_widget (ch_dlg, button_new (XTRACT (i)));
+	    add_widget (ch_dlg,
+			button_new (BY + chmod_but[i].y - single_set,
+				    BX + chmod_but[i].x,
+				    chmod_but[i].ret_cmd,
+				    chmod_but[i].flags,
+				    _(chmod_but[i].text), 0));
     }
 
-
-#define XTRACT2(i) 0, _(check_perm [i].text), NULL
     for (i = 0; i < PERMISSIONS; i++) {
-	check_perm[i].check = check_new (PY + (PERMISSIONS - i), PX + 2,
-					 XTRACT2 (i));
+	check_perm[i].check =
+	    check_new (PY + (PERMISSIONS - i), PX + 2, 0,
+		       _(check_perm[i].text));
 	add_widget (ch_dlg, check_perm[i].check);
     }
-}
 
-int stat_file (char *filename, struct stat *st)
-{
-    if (mc_stat (filename, st))
-	return 0;
-
-    return 1;
+    return ch_dlg;
 }
 
 static void chmod_done (void)
@@ -255,21 +232,21 @@ static void chmod_done (void)
 
 static char *next_file (void)
 {
-    while (!cpanel->dir.list[c_file].f.marked)
+    while (!current_panel->dir.list[c_file].f.marked)
 	c_file++;
 
-    return cpanel->dir.list[c_file].fname;
+    return current_panel->dir.list[c_file].fname;
 }
 
 static void do_chmod (struct stat *sf)
 {
     sf->st_mode &= and_mask;
     sf->st_mode |= or_mask;
-    if (mc_chmod (cpanel->dir.list [c_file].fname, sf->st_mode) == -1)
-	message (1, MSG_ERROR, _(" Couldn't chmod \"%s\" \n %s "),
-	     cpanel->dir.list [c_file].fname, unix_error_string (errno));
+    if (mc_chmod (current_panel->dir.list [c_file].fname, sf->st_mode) == -1)
+	message (1, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
+	     current_panel->dir.list [c_file].fname, unix_error_string (errno));
 
-    do_file_mark (cpanel, c_file, 0);
+    do_file_mark (current_panel, c_file, 0);
 }
 
 static void apply_mask (struct stat *sf)
@@ -281,12 +258,12 @@ static void apply_mask (struct stat *sf)
 
     do {
 	fname = next_file ();
-	if (!stat_file (fname, sf))
+	if (mc_stat (fname, sf) != 0)
 	    return;
 	c_stat = sf->st_mode;
 
 	do_chmod (sf);
-    } while (cpanel->marked);
+    } while (current_panel->marked);
 }
 
 void chmod_cmd (void)
@@ -295,15 +272,16 @@ void chmod_cmd (void)
     char *fname;
     int i;
     struct stat sf_stat;
+    Dlg_head *ch_dlg;
 
     do {			/* do while any files remaining */
-	init_chmod ();
-	if (cpanel->marked)
+	ch_dlg = init_chmod ();
+	if (current_panel->marked)
 	    fname = next_file ();	/* next marked file */
 	else
-	    fname = selection (cpanel)->fname;	/* single file */
+	    fname = selection (current_panel)->fname;	/* single file */
 
-	if (!stat_file (fname, &sf_stat)){	/* get status of file */
+	if (mc_stat (fname, &sf_stat) != 0) {	/* get status of file */
 	    destroy_dlg (ch_dlg);
 	    break;
 	}
@@ -319,13 +297,13 @@ void chmod_cmd (void)
 
 	/* Set the labels */
 	c_fname = name_trunc (fname, 21);
-	add_widget (ch_dlg, label_new (FY+2, FX+2, c_fname, NULL));
+	add_widget (ch_dlg, label_new (FY+2, FX+2, c_fname));
 	c_fown = name_trunc (get_owner (sf_stat.st_uid), 21);
-	add_widget (ch_dlg, label_new (FY+6, FX+2, c_fown, NULL));
+	add_widget (ch_dlg, label_new (FY+6, FX+2, c_fown));
 	c_fgrp = name_trunc (get_group (sf_stat.st_gid), 21);
-	add_widget (ch_dlg, label_new (FY+8, FX+2, c_fgrp, NULL));
+	add_widget (ch_dlg, label_new (FY+8, FX+2, c_fgrp));
 	g_snprintf (buffer, sizeof (buffer), "%o", c_stat);
-	statl = label_new (FY+4, FX+2, buffer, NULL);
+	statl = label_new (FY+4, FX+2, buffer);
 	add_widget (ch_dlg, statl);
 	
 	run_dlg (ch_dlg);	/* retrieve an action */
@@ -335,7 +313,7 @@ void chmod_cmd (void)
 	case B_ENTER:
 	    if (mode_change)
 		if (mc_chmod (fname, c_stat) == -1)
-		    message (1, MSG_ERROR, _(" Couldn't chmod \"%s\" \n %s "),
+		    message (1, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
 	 		 fname, unix_error_string (errno));
 	    need_update = 1;
 	    break;
@@ -385,28 +363,12 @@ void chmod_cmd (void)
 	    break;
 	}
 
-	if (cpanel->marked && ch_dlg->ret_value!=B_CANCEL) {
-	    do_file_mark (cpanel, c_file, 0);
+	if (current_panel->marked && ch_dlg->ret_value!=B_CANCEL) {
+	    do_file_mark (current_panel, c_file, 0);
     	    need_update = 1;
 	}
 	destroy_dlg (ch_dlg);
-    } while (cpanel->marked && !end_chmod);
+    } while (current_panel->marked && !end_chmod);
     chmod_done ();
 }
 
-void ch1_cmd (int id)
-{
-  if (advanced_chfns)
-      chown_advanced_cmd ();
-  else
-      chmod_cmd ();
-}
-
-void ch2_cmd (int id)
-{
-  if (advanced_chfns)
-      chown_advanced_cmd ();
-  else
-      chown_cmd ();
-}
-
