@@ -17,11 +17,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #endif
@@ -33,14 +34,14 @@
 #include "tty.h"
 #include "win.h"
 #include "color.h"
-#include "dlg.h"
-#include "widget.h"
 #include "dialog.h"
+#include "widget.h"
 #include "wtools.h"
 #include "complete.h"
 #include "main.h"
 #include "key.h"		/* XCTRL and ALT macros */
-#include "../vfs/vfs.h"
+
+typedef char *CompletionFunction (char *, int);
 
 /* This flag is used in filename_completion_function */
 static int ignore_filenames = 0;
@@ -56,21 +57,18 @@ filename_completion_function (char *text, int state)
     static char *filename = NULL;
     static char *dirname = NULL;
     static char *users_dirname = NULL;
-    static int filename_len;
+    static size_t filename_len;
     int isdir = 1, isexec = 0;
 
     struct dirent *entry = NULL;
 
     /* If we're starting the match process, initialize us a bit. */
     if (!state){
-        char *temp;
+        const char *temp;
 
-        if (dirname)
-            g_free (dirname);
-        if (filename) 
-            g_free (filename);
-        if (users_dirname)
-            g_free (users_dirname);
+        g_free (dirname);
+        g_free (filename);
+        g_free (users_dirname);
 
 	if ((*text) && (temp = strrchr (text, PATH_SEP))){
 	    filename = g_strdup (++temp);
@@ -86,12 +84,6 @@ filename_completion_function (char *text, int state)
         users_dirname = dirname;
         {
 	    dirname = tilde_expand (dirname);
-	    if (!dirname){
-		g_free (users_dirname);
-		g_free (filename);
-		users_dirname = filename = NULL;
-		return NULL;
-	    }
 	    canonicalize_pathname (dirname);
 	    /* Here we should do something with variable expansion
 	       and `command`.
@@ -161,18 +153,12 @@ filename_completion_function (char *text, int state)
 	    mc_closedir (directory);
 	    directory = NULL;
 	}
-        if (dirname){
-	    g_free (dirname);
-	    dirname = NULL;
-	}
-        if (filename){
-	    g_free (filename);
-	    filename = NULL;
-	}
-        if (users_dirname){
-	    g_free (users_dirname);
-	    users_dirname = NULL;
-	}
+	g_free (dirname);
+	dirname = NULL;
+	g_free (filename);
+	filename = NULL;
+	g_free (users_dirname);
+	users_dirname = NULL;
         return NULL;
     } else {
         char *temp;
@@ -199,12 +185,6 @@ filename_completion_function (char *text, int state)
 
 /* We assume here that text[0] == '~' , if you want to call it in another way,
    you have to change the code */
-#ifdef NATIVE_WIN32
-char *username_completion_function (char *text, int state)
-{
-    return NULL;
-}
-#else
 static char *
 username_completion_function (char *text, int state)
 {
@@ -237,8 +217,10 @@ username_completion_function (char *text, int state)
     }
 }
 
+/* Linux declares environ in <unistd.h>, so don't repeat it here. */
+#if (!(defined(__linux__) && defined (__USE_GNU)) && !defined(__CYGWIN__))
 extern char **environ;
-#endif /* NATIVE_WIN32 */
+#endif
 
 /* We assume text [0] == '$' and want to have a look at text [1], if it is
    equal to '{', so that we should append '}' at the end */
@@ -247,7 +229,7 @@ variable_completion_function (char *text, int state)
 {
     static char **env_p;
     static int varlen, isbrace;
-    char *p = 0;
+    const char *p = NULL;
 
     if (!state){ /* Initialization stuff */
 	isbrace = (text [1] == '{');
@@ -270,7 +252,7 @@ variable_completion_function (char *text, int state)
 	*temp = '$';
 	if (isbrace)
 	    temp [1] = '{';
-        strncpy (temp + 1 + isbrace, *env_p, p - *env_p);
+	memcpy (temp + 1 + isbrace, *env_p, p - *env_p);
         if (isbrace)
             strcpy (temp + 2 + (p - *env_p), "}");
         else
@@ -286,7 +268,7 @@ variable_completion_function (char *text, int state)
 static char **hosts = NULL;
 static char **hosts_p = NULL;
 static int hosts_alloclen = 0;
-static void fetch_hosts (char *filename)
+static void fetch_hosts (const char *filename)
 {
     FILE *file = fopen (filename, "r");
     char *temp, buffer[256], *name;
@@ -364,7 +346,7 @@ hostname_completion_function (char *text, int state)
     static int textstart, textlen;
 
     if (!state){ /* Initialization stuff */
-        char *p;
+        const char *p;
         
     	if (hosts != NULL){
     	    for (host_p = hosts; *host_p; host_p++)
@@ -414,116 +396,113 @@ hostname_completion_function (char *text, int state)
 static char *
 command_completion_function (char *text, int state)
 {
-    static char *path_end;
+    static const char *path_end;
     static int isabsolute;
     static int phase;
     static int text_len;
-    static const char * const * words;
+    static const char *const *words;
     static char *path;
     static char *cur_path;
     static char *cur_word;
     static int init_state;
-    static const char * const bash_reserved [] = { 
-	"if", "then", "else", "elif", "fi", "case", "esac", "for", "select",
-	"while", "until", "do", "done", "in", "function" , 0
+    static const char *const bash_reserved[] = {
+	"if", "then", "else", "elif", "fi", "case", "esac", "for",
+	    "select", "while", "until", "do", "done", "in", "function", 0
     };
-    static const char * const bash_builtins [] = { 
-	"alias", "bg", "bind", "break", "builtin", "cd", "command", "continue",
-	"declare", "dirs", "echo", "enable", "eval", "exec", "exit", "export",
-	"fc", "fg", "getopts", "hash", "help", "history", "jobs", "kill", "let",
-	"local", "logout", "popd", "pushd", "pwd", "read", "readonly", "return",
-	"set", "shift", "source", "suspend", "test", "times", "trap", "type", 
-	"typeset", "ulimit", "umask", "unalias", "unset", "wait" , 0
+    static const char *const bash_builtins[] = {
+	"alias", "bg", "bind", "break", "builtin", "cd", "command",
+	    "continue", "declare", "dirs", "echo", "enable", "eval",
+	    "exec", "exit", "export", "fc", "fg", "getopts", "hash",
+	    "help", "history", "jobs", "kill", "let", "local", "logout",
+	    "popd", "pushd", "pwd", "read", "readonly", "return", "set",
+	    "shift", "source", "suspend", "test", "times", "trap", "type",
+	    "typeset", "ulimit", "umask", "unalias", "unset", "wait", 0
     };
     char *p, *found;
 
-    if (!state){ /* Initialize us a little bit */
+    if (!state) {		/* Initialize us a little bit */
 	isabsolute = strchr (text, PATH_SEP) != 0;
-        look_for_executables = isabsolute ? 1 : 2;
-	if (!isabsolute){
+	look_for_executables = isabsolute ? 1 : 2;
+	if (!isabsolute) {
 	    words = bash_reserved;
 	    phase = 0;
 	    text_len = strlen (text);
-	    path = getenv ("PATH");
-	    if (path){
-		p = path = g_strdup (path);
+	    if (!path && (path = g_strdup (getenv ("PATH"))) != NULL) {
+		p = path;
 		path_end = strchr (p, 0);
-		while ((p = strchr (p, PATH_ENV_SEP))){
+		while ((p = strchr (p, PATH_ENV_SEP))) {
 		    *p++ = 0;
-	    	}
+		}
 	    }
 	}
     }
-    
-    if (isabsolute){
-        p = filename_completion_function (text, state);
-        if (!p)
-            look_for_executables = 0;
-        return p;
+
+    if (isabsolute) {
+	p = filename_completion_function (text, state);
+	if (!p)
+	    look_for_executables = 0;
+	return p;
     }
 
-    found = NULL;    
-    switch (phase){
-    	case 0: /* Reserved words */
-	    while (*words){
-	        if (!strncmp (*words, text, text_len))
-	            return g_strdup (*(words++));
-	        words++;
+    found = NULL;
+    switch (phase) {
+    case 0:			/* Reserved words */
+	while (*words) {
+	    if (!strncmp (*words, text, text_len))
+		return g_strdup (*(words++));
+	    words++;
+	}
+	phase++;
+	words = bash_builtins;
+    case 1:			/* Builtin commands */
+	while (*words) {
+	    if (!strncmp (*words, text, text_len))
+		return g_strdup (*(words++));
+	    words++;
+	}
+	phase++;
+	if (!path)
+	    break;
+	cur_path = path;
+	cur_word = NULL;
+    case 2:			/* And looking through the $PATH */
+	while (!found) {
+	    if (!cur_word) {
+		char *expanded;
+
+		if (cur_path >= path_end)
+		    break;
+		expanded = tilde_expand (*cur_path ? cur_path : ".");
+		cur_word = concat_dir_and_file (expanded, text);
+		g_free (expanded);
+		canonicalize_pathname (cur_word);
+		cur_path = strchr (cur_path, 0) + 1;
+		init_state = state;
 	    }
-	    phase++;
-	    words = bash_builtins;
-	case 1: /* Builtin commands */
-	    while (*words){
-	        if (!strncmp (*words, text, text_len))
-	            return g_strdup (*(words++));
-	        words++;
-	    }
-	    phase++;
-	    if (!path)
-	        break;
-	    cur_path = path;
-	    cur_word = NULL;
-	case 2: /* And looking through the $PATH */
-	    while (!found){
-	        if (!cur_word){
-		    char *expanded;
-		    
-		    if (cur_path >= path_end)
-	            	break;
-		    expanded = tilde_expand (*cur_path ? cur_path : ".");
-		    if (!expanded){
-			g_free (path);
-			path = NULL;
-			return NULL;
-		    }
-	            p = canonicalize_pathname (expanded);
-		    cur_word = concat_dir_and_file (p, text);
-	            g_free (p);
-	            cur_path = strchr (cur_path, 0) + 1;
-	            init_state = state;
-	        }
-	        found = filename_completion_function (cur_word, state - init_state);
-	        if (!found){
-	            g_free (cur_word);
-	            cur_word = NULL;
-	        }
+	    found =
+		filename_completion_function (cur_word,
+					      state - init_state);
+	    if (!found) {
+		g_free (cur_word);
+		cur_word = NULL;
 	    }
+	}
     }
-    
-    if (!found){
-        look_for_executables = 0;
-        if (path)
-            g_free (path);
-        return NULL;
+
+    if (!found) {
+	look_for_executables = 0;
+	g_free (path);
+	path = NULL;
+	return NULL;
     }
-    if ((p = strrchr (found, PATH_SEP)) != NULL){
-        p++;
-        p = g_strdup (p);
-        g_free (found);
-        return p;
+    if ((p = strrchr (found, PATH_SEP)) != NULL) {
+	p++;
+	p = g_strdup (p);
+	g_free (found);
+	return p;
     }
     return found;
-    
+
 }
 
 static int
@@ -573,7 +552,7 @@ completion_matches (char *text, CompletionFunction entry_function)
         /* If only one match, just use that. */
         if (matches == 1){
 	    match_list[0] = match_list[1];
-	    match_list[1] = (char *)NULL;
+	    match_list[1] = NULL;
         } else {
             int j;
             
@@ -596,6 +575,7 @@ completion_matches (char *text, CompletionFunction entry_function)
 		    j++;
 		    if (j > matches)
 		        break;
+		    continue; /* Look for a run of equal strings */
 		} else
 	            if (low > si) low = si;
 		if (i + 1 != j) /* So there's some gap */
@@ -604,9 +584,7 @@ completion_matches (char *text, CompletionFunction entry_function)
 	    }
 	    matches = i;
             match_list [matches + 1] = NULL;
-	    match_list[0] = g_malloc (low + 1);
-	    strncpy (match_list[0], match_list[1], low);
-	    match_list[0][low] = 0;
+	    match_list[0] = g_strndup(match_list[1], low);
 	}
     } else {				/* There were no matches. */
         g_free (match_list);
@@ -615,17 +593,28 @@ completion_matches (char *text, CompletionFunction entry_function)
     return match_list;
 }
 
+/* Check if directory completion is needed */
 static int
-check_is_cd (char *text, int start, int flags)
+check_is_cd (const char *text, int start, int flags)
 {
-    char *p, *q = text + start;
-    	    
-    for (p = text; *p && p < q && (*p == ' ' || *p == '\t'); p++);
-    if (((flags & INPUT_COMPLETE_COMMANDS) && 
-        !strncmp (p, "cd", 2) && (p [2] == ' ' || p [2] == '\t') && 
-        p + 2 < q) ||
-        (flags & INPUT_COMPLETE_CD))
-        return 1;
+    const unsigned char *p, *q;
+
+    if (flags & INPUT_COMPLETE_CD)
+	return 1;
+
+    if (!(flags & INPUT_COMPLETE_COMMANDS))
+	return 0;
+
+    /* Skip initial spaces */
+    p = (const unsigned char *) text;
+    q = (const unsigned char *) text + start;
+    while (p < q && *p && isspace (*p))
+	p++;
+
+    /* Check if the command is "cd" and the cursor is after it */
+    if (p[0] == 'c' && p[1] == 'd' && isspace (p[2]) && (p + 2 < q))
+	return 1;
+
     return 0;
 }
 
@@ -636,7 +625,7 @@ try_complete (char *text, int *start, int *end, int flags)
     int in_command_position = 0, i;
     char *word, c;
     char **matches = NULL;
-    char *command_separator_chars = ";|&{(`";
+    const char *command_separator_chars = ";|&{(`";
     char *p = NULL, *q = NULL, *r = NULL;
     int is_cd = check_is_cd (text, *start, flags);
 
@@ -734,7 +723,8 @@ try_complete (char *text, int *start, int *end, int flags)
     	    if (!strncmp (p, "cd", 2))
     	        for (p += 2; *p && p < q && (*p == ' ' || *p == '\t'); p++);
     	    if (p == q){
-		char *cdpath = getenv ("CDPATH");
+		char * const cdpath_ref = g_strdup (getenv ("CDPATH"));
+		char *cdpath = cdpath_ref;
 		char c, *s, *r;
 
 		if (cdpath == NULL)
@@ -757,12 +747,12 @@ try_complete (char *text, int *start, int *end, int flags)
 		    *s = c;
 		    cdpath = s + 1;
 		}
+		g_free (cdpath_ref);
     	    }
     	}
     }
 
-    if (word)
-    	g_free (word);
+    g_free (word);
 
     return matches;
 }
@@ -784,10 +774,10 @@ static WInput *input;
 static int min_end;
 static int start, end;
 
-static int insert_text (WInput *in, char *text, int len)
+static int insert_text (WInput *in, char *text, ssize_t len)
 {
-    len = min (len, strlen (text)) + start - end;
-    if (strlen (in->buffer) + len >= in->current_max_len){
+    len = min (len, (ssize_t) strlen (text)) + start - end;
+    if (strlen (in->buffer) + len >= (size_t) in->current_max_len){
     /* Expand the buffer */
     	char *narea = g_realloc (in->buffer, in->current_max_len + len + in->field_len);
 	if (narea){
@@ -795,7 +785,7 @@ static int insert_text (WInput *in, char *text, int len)
 	    in->current_max_len += len + in->field_len;
 	}
     }
-    if (strlen (in->buffer)+1 < in->current_max_len){
+    if (strlen (in->buffer)+1 < (size_t) in->current_max_len){
     	if (len > 0){
 	    int i = strlen (&in->buffer [end]);
 	    for (; i >= 0; i--)
@@ -806,7 +796,7 @@ static int insert_text (WInput *in, char *text, int len)
 	    	*(p++) = *(q++);
 	    *p = 0;
 	}
-	strncpy (in->buffer + start, text, len - start + end);
+	memcpy (in->buffer + start, text, len - start + end);
 	in->point += len;
 	update_input (in, 1);
 	end += len;
@@ -814,25 +804,19 @@ static int insert_text (WInput *in, char *text, int len)
     return len != 0;
 }
 
-static int
-query_callback (Dlg_head * h, int Par, int Msg)
+static cb_ret_t
+query_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    switch (Msg) {
-    case DLG_DRAW:
-	common_dialog_repaint (h);
-	break;
-
+    switch (msg) {
     case DLG_KEY:
-	switch (Par) {
+	switch (parm) {
 	case KEY_LEFT:
 	case KEY_RIGHT:
 	    h->ret_value = 0;
 	    dlg_stop (h);
 	    return 1;
 
-	case 0177:
 	case KEY_BACKSPACE:
-	case XCTRL ('h'):
 	    if (end == min_end) {
 		h->ret_value = 0;
 		dlg_stop (h);
@@ -840,16 +824,16 @@ query_callback (Dlg_head * h, int Par, int Msg)
 	    } else {
 		WLEntry *e, *e1;
 
-		e1 = e = ((WListbox *) (h->current->widget))->list;
+		e1 = e = ((WListbox *) (h->current))->list;
 		do {
 		    if (!strncmp
 			(input->buffer + start, e1->text,
 			 end - start - 1)) {
-			listbox_select_entry ((WListbox *) (h->current->
-							    widget), e1);
-			handle_char (input, Par);
+			listbox_select_entry ((WListbox *) (h->current),
+					      e1);
+			handle_char (input, parm);
 			end--;
-			send_message (h->current->widget, WIDGET_DRAW, 0);
+			send_message (h->current, WIDGET_DRAW, 0);
 			break;
 		    }
 		    e1 = e1->next;
@@ -858,8 +842,8 @@ query_callback (Dlg_head * h, int Par, int Msg)
 	    return 1;
 
 	default:
-	    if (Par > 0xff || !is_printable (Par)) {
-		if (is_in_input_map (input, Par) == 2) {
+	    if (parm > 0xff || !is_printable (parm)) {
+		if (is_in_input_map (input, parm) == 2) {
 		    if (end == min_end)
 			return 1;
 		    h->ret_value = B_USER;	/* This means we want to refill the
@@ -874,17 +858,17 @@ query_callback (Dlg_head * h, int Par, int Msg)
 		int low = 4096;
 		char *last_text = NULL;
 
-		e1 = e = ((WListbox *) (h->current->widget))->list;
+		e1 = e = ((WListbox *) (h->current))->list;
 		do {
 		    if (!strncmp
 			(input->buffer + start, e1->text, end - start)) {
-			if (e1->text[end - start] == Par) {
+			if (e1->text[end - start] == parm) {
 			    if (need_redraw) {
 				register int c1, c2, si;
 
 				for (si = end - start + 1;
-				     (c1 = last_text[si]) &&
-				     (c2 = e1->text[si]); si++)
+				     (c1 = last_text[si])
+				     && (c2 = e1->text[si]); si++)
 				    if (c1 != c2)
 					break;
 				if (low > si)
@@ -894,8 +878,7 @@ query_callback (Dlg_head * h, int Par, int Msg)
 			    } else {
 				need_redraw = 1;
 				listbox_select_entry ((WListbox *) (h->
-								    current->
-								    widget),
+								    current),
 						      e1);
 				last_text = e1->text;
 			    }
@@ -905,22 +888,19 @@ query_callback (Dlg_head * h, int Par, int Msg)
 		} while (e != e1);
 		if (need_redraw == 2) {
 		    insert_text (input, last_text, low);
-		    send_message (h->current->widget, WIDGET_DRAW, 0);
+		    send_message (h->current, WIDGET_DRAW, 0);
 		} else if (need_redraw == 1) {
 		    h->ret_value = B_ENTER;
 		    dlg_stop (h);
 		}
 	    }
-	    return 1;
+	    return MSG_HANDLED;
 	}
 	break;
-    }
-    return 0;
-}
 
-static int querylist_callback (void *data)
-{
-    return 1;
+    default:
+	return default_dlg_callback (h, msg, parm);
+    }
 }
 
 #define DO_INSERTION 1
@@ -941,7 +921,7 @@ complete_engine (WInput *in, int what_to_do)
     	in->completions = try_complete (in->buffer, &start, &end, in->completion_flags);
     }
     if (in->completions){
-    	if (what_to_do & DO_INSERTION) {
+    	if (what_to_do & DO_INSERTION || ((what_to_do & DO_QUERY) && !in->completions[1])) {
     	    if (insert_text (in, in->completions [0], strlen (in->completions [0]))){
     	        if (in->completions [1])
     	    	    beep ();
@@ -950,11 +930,7 @@ complete_engine (WInput *in, int what_to_do)
 	    } else
 	        beep ();
         }
-	/* FIXME: evil evil evil.  We do not go into the query completion engine
-	 * because we do not have a Gtk dialog for it.  Gtk-ted does not like
-	 * this; if we enable this code, it will crash.
-	 */
-    	if ((what_to_do & DO_QUERY) && in->completions [1]) {
+    	if ((what_to_do & DO_QUERY) && in->completions && in->completions [1]) {
     	    int maxlen = 0, i, count = 0;
     	    int x, y, w, h;
     	    int start_x, start_y;
@@ -994,7 +970,7 @@ complete_engine (WInput *in, int what_to_do)
     	    query_dlg = create_dlg (y, x, query_height, query_width,
 				    dialog_colors, query_callback,
 				    "[Completion]", NULL, DLG_COMPACT);
-    	    query_list = listbox_new (1, 1, w - 2, h - 2, 0, querylist_callback, NULL);
+    	    query_list = listbox_new (1, 1, w - 2, h - 2, NULL);
     	    add_widget (query_dlg, query_list);
     	    for (p = in->completions + 1; *p; p++)
     	    	listbox_add_item (query_list, 0, 0, *p, NULL);
@@ -1019,11 +995,17 @@ complete_engine (WInput *in, int what_to_do)
 
 void complete (WInput *in)
 {
+    int engine_flags;
+
     if (in->completions)
-    	while (complete_engine (in, DO_QUERY));
-    else if (show_all_if_ambiguous){
-    	complete_engine (in, DO_INSERTION);
-    	while (complete_engine (in, DO_QUERY));
-    } else
-    	complete_engine (in, DO_INSERTION);
+	engine_flags = DO_QUERY;
+    else
+    {
+	engine_flags = DO_INSERTION;
+
+	if (show_all_if_ambiguous)
+	    engine_flags |= DO_QUERY;
+    }
+
+    while (complete_engine (in, engine_flags));
 }
