@@ -13,38 +13,26 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#define DIR_H_INCLUDE_HANDLE_DIRENT
-#include "tty.h"
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/stat.h>
-#include "x.h"
+
 #include "global.h"
+#include "tty.h"
 #include "dir.h"
-#include "dialog.h"
-#include "tree.h"
-#include "../vfs/vfs.h"
+#include "wtools.h"
+#include "treestore.h"
 
-/* "$Id: dir.c,v 1.25 2001/08/12 16:36:55 proskin Exp $" */
-
-/*
- * If true show files starting with a dot.
- *
- * In GNOME we default to not showing them.
- */
-#ifdef HAVE_GNOME
-int show_dot_files = 0;
-#else
+/* If true show files starting with a dot */
 int show_dot_files = 1;
-#endif
 
 /* If true show files ending in ~ */
-int show_backups = 0;
+int show_backups = 1;
 
 /* If false then directories are shown separately from files */
 int mix_all_files = 0;
@@ -55,7 +43,7 @@ static int reverse = 1;
 /* Are the files sorted case sensitively? */
 static int case_sensitive = OS_SORT_CASE_SENSITIVE_DEFAULT;
 
-#define MY_ISDIR(x) ( (S_ISDIR (x->buf.st_mode) || x->f.link_to_dir) ? 1 : 0)
+#define MY_ISDIR(x) ( (S_ISDIR (x->st.st_mode) || x->f.link_to_dir) ? 1 : 0)
 
 sort_orders_t sort_orders [SORT_TYPES_TOTAL] = {
     { N_("&Unsorted"),    unsorted },
@@ -91,8 +79,7 @@ typedef enum {
     STRCOLL_TEST	
 } strcoll_status;
 
-static int
-string_sortcomp (char *str1, char *str2)
+static int string_sortcomp (const char *str1, const char *str2)
 {
     static strcoll_status use_strcoll = STRCOLL_TEST;
 
@@ -139,7 +126,7 @@ sort_name (const file_entry *a, const file_entry *b)
 int
 sort_ext (const file_entry *a, const file_entry *b)
 {
-    char *exta, *extb;
+    const char *exta, *extb;
     int r;
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
@@ -163,7 +150,7 @@ sort_owner (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return string_sortcomp (get_owner (a->buf.st_uid), get_owner (a->buf.st_uid)) * reverse;
+	return string_sortcomp (get_owner (a->st.st_uid), get_owner (a->st.st_uid)) * reverse;
     return bd-ad;
 }
 
@@ -174,7 +161,7 @@ sort_group (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return string_sortcomp (get_group (a->buf.st_gid), get_group (a->buf.st_gid)) * reverse;
+	return string_sortcomp (get_group (a->st.st_gid), get_group (a->st.st_gid)) * reverse;
     return bd-ad;
 }
 
@@ -185,7 +172,7 @@ sort_time (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return (a->buf.st_mtime - b->buf.st_mtime) * reverse;
+	return (a->st.st_mtime - b->st.st_mtime) * reverse;
     else
 	return bd-ad;
 }
@@ -197,7 +184,7 @@ sort_ctime (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return (a->buf.st_ctime - b->buf.st_ctime) * reverse;
+	return (a->st.st_ctime - b->st.st_ctime) * reverse;
     else
 	return bd-ad;
 }
@@ -209,7 +196,7 @@ sort_atime (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return (a->buf.st_atime - b->buf.st_atime) * reverse;
+	return (a->st.st_atime - b->st.st_atime) * reverse;
     else
 	return bd-ad;
 }
@@ -221,7 +208,7 @@ sort_inode (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return (a->buf.st_ino - b->buf.st_ino) * reverse;
+	return (a->st.st_ino - b->st.st_ino) * reverse;
     else
 	return bd-ad;
 }
@@ -232,10 +219,10 @@ sort_size (const file_entry *a, const file_entry *b)
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
 
-    if (ad == bd || mix_all_files)
-	return (b->buf.st_size - a->buf.st_size) * reverse;
-    else
-	return bd-ad;
+    if (ad != bd && !mix_all_files)
+	return bd - ad;
+
+    return (2 * (b->st.st_size > a->st.st_size) - 1) * reverse;
 }
 
 int
@@ -245,7 +232,7 @@ sort_links (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return (b->buf.st_nlink - a->buf.st_nlink) * reverse;
+	return (b->st.st_nlink - a->st.st_nlink) * reverse;
     else
 	return bd-ad;
 }
@@ -257,7 +244,7 @@ sort_ngid (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return (b->buf.st_gid - a->buf.st_gid) * reverse;
+	return (b->st.st_gid - a->st.st_gid) * reverse;
     else
 	return bd-ad;
 }
@@ -269,7 +256,7 @@ sort_nuid (const file_entry *a, const file_entry *b)
     int bd = MY_ISDIR (b);
 
     if (ad == bd || mix_all_files)
-	return (b->buf.st_uid - a->buf.st_uid) * reverse;
+	return (b->st.st_uid - a->st.st_uid) * reverse;
     else
 	return bd-ad;
 }
@@ -277,14 +264,14 @@ sort_nuid (const file_entry *a, const file_entry *b)
 inline static int
 file_type_to_num (const file_entry *fe)
 {
-    const struct stat *s = &fe->buf;
+    const struct stat *s = &fe->st;
 
     if (S_ISDIR (s->st_mode))
 	return 0;
     if (S_ISLNK (s->st_mode)){
 	if (fe->f.link_to_dir)
 	    return 1;
-	if (fe->f.stalled_link)
+	if (fe->f.stale_link)
 	    return 2;
 	else
 	    return 3;
@@ -297,6 +284,8 @@ file_type_to_num (const file_entry *fe)
 	return 6;
     if (S_ISFIFO (s->st_mode))
 	return 7;
+    if (S_ISNAM (s->st_mode)) /* Special named files will be shown as block devices */
+	return 6;
     if (is_exe (s->st_mode))
 	return 8;
     return 9;
@@ -316,10 +305,12 @@ void
 do_sort (dir_list *list, sortfn *sort, int top, int reverse_f, int case_sensitive_f)
 {
     int i;
+    int dot_dot_found = 0;
     file_entry tmp_fe;
 
     for (i = 0; i < top + 1; i++) {             /* put ".." first in list */
 	if (!strcmp (list->list [i].fname, "..")) {
+	    dot_dot_found = 1;
             if (i > 0) {                        /* swap [i] and [0] */
                 memcpy (&tmp_fe, &(list->list [0]), sizeof (file_entry));
                 memcpy (&(list->list [0]), &(list->list [i]), sizeof (file_entry));
@@ -331,7 +322,8 @@ do_sort (dir_list *list, sortfn *sort, int top, int reverse_f, int case_sensitiv
 
     reverse = reverse_f ? -1 : 1;
     case_sensitive = case_sensitive_f;
-    qsort (&(list->list) [1], top, sizeof (file_entry), sort);
+    qsort (&(list->list) [dot_dot_found],
+	   top + 1 - dot_dot_found, sizeof (file_entry), sort);
 }
 
 void
@@ -348,10 +340,6 @@ clean_dir (dir_list *list, int count)
 static int
 add_dotdot_to_list (dir_list *list, int index)
 {
-    char buffer [MC_MAXPATHLEN + MC_MAXPATHLEN];
-    char *p;
-    int i = 0;
-
     /* Need to grow the *list? */
     if (index == list->size) {
 	list->list = g_realloc (list->list, sizeof (file_entry) *
@@ -361,40 +349,14 @@ add_dotdot_to_list (dir_list *list, int index)
 	list->size += RESIZE_STEPS;
     }
 
+    memset (&(list->list) [index], 0, sizeof(file_entry));
     (list->list) [index].fnamelen = 2;
     (list->list) [index].fname = g_strdup ("..");
     (list->list) [index].f.link_to_dir = 0;
-    (list->list) [index].f.stalled_link = 0;
+    (list->list) [index].f.stale_link = 0;
     (list->list) [index].f.dir_size_computed = 0;
-
-    /* FIXME: We need to get the panel definition! to use file_mark */
     (list->list) [index].f.marked = 0;
-    mc_get_current_wd (buffer, sizeof (buffer) - 1 );
-    if (buffer [strlen (buffer) - 1] == PATH_SEP)
-    	buffer [strlen (buffer) - 1] = 0;
-    for (;;) {
-    	strcat (buffer, PATH_SEP_STR "..");
-        p = vfs_canon (buffer);
-        if (mc_stat (p, &((list->list) [index].buf)) != -1){
-	    g_free (p);
-            break;
-	}
-        i = 1;
-        if (!strcmp (p, PATH_SEP_STR)){
-	    g_free (p);
-            return 1;
-	}
-	strcpy (buffer, p);
-	g_free (p);
-    }
-
-/* Commented out to preserve a usable '..'. What's the purpose of this
- * three lines? (Norbert) */
-#if 0
-    if (i) { /* So there is bogus information on the .. directory's stat */
-        (list->list) [index].buf.st_mode &= ~0444;
-    }
-#endif
+    (list->list) [index].st.st_mode = 040755;
     return 1;
 }
 
@@ -407,25 +369,26 @@ set_zero_dir (dir_list *list)
 
 /* If you change handle_dirent then check also handle_path. */
 /* Return values: -1 = failure, 0 = don't add, 1 = add to the list */
-int
-handle_dirent (dir_list *list, char *filter, struct dirent *dp,
+static int
+handle_dirent (dir_list *list, const char *filter, struct dirent *dp,
 	       struct stat *buf1, int next_free, int *link_to_dir,
-	       int *stalled_link)
+	       int *stale_link)
 {
-    if (dp->d_name [0] == '.' && dp->d_name [1] == 0)
+    if (dp->d_name[0] == '.' && dp->d_name[1] == 0)
 	return 0;
-    if (!show_dot_files){
-	if (dp->d_name [0] == '.'){
-	    if (!(dp->d_name [1] == 0))
-		if (!(dp->d_name [1] == '.' && NLENGTH (dp) == 2))
-		    return 0;
-	}
-    }
-    if (!show_backups && dp->d_name [NLENGTH (dp)-1] == '~')
+    if (dp->d_name[0] == '.' && dp->d_name[1] == '.' && dp->d_name[2] == 0)
+	return 0;
+    if (!show_dot_files && (dp->d_name[0] == '.'))
+	return 0;
+    if (!show_backups && dp->d_name[NLENGTH (dp) - 1] == '~')
 	return 0;
     if (mc_lstat (dp->d_name, buf1) == -1) {
-	message(1, _(" Error "), _("File '%s' exists but can not be stat-ed: %s"), dp->d_name, strerror(errno));
-        return 0;
+	/*
+	 * lstat() fails - such entries should be identified by
+	 * buf1->st_mode being 0.
+	 * It happens on QNX Neutrino for /fs/cd0 if no CD is inserted.
+	 */
+	memset (buf1, 0, sizeof (*buf1));
     }
 
     if (S_ISDIR (buf1->st_mode))
@@ -433,22 +396,23 @@ handle_dirent (dir_list *list, char *filter, struct dirent *dp,
 
     /* A link to a file or a directory? */
     *link_to_dir = 0;
-    *stalled_link = 0;
-    if (S_ISLNK(buf1->st_mode)){
+    *stale_link = 0;
+    if (S_ISLNK (buf1->st_mode)) {
 	struct stat buf2;
 	if (!mc_stat (dp->d_name, &buf2))
-	    *link_to_dir = S_ISDIR(buf2.st_mode) != 0;
+	    *link_to_dir = S_ISDIR (buf2.st_mode) != 0;
 	else
-	    *stalled_link = 1;
+	    *stale_link = 1;
     }
-    if (!(S_ISDIR(buf1->st_mode) || *link_to_dir) && filter &&
-	!regexp_match (filter, dp->d_name, match_file))
+    if (!(S_ISDIR (buf1->st_mode) || *link_to_dir) && filter
+	&& !regexp_match (filter, dp->d_name, match_file))
 	return 0;
 
     /* Need to grow the *list? */
-    if (next_free == list->size){
-	list->list = g_realloc (list->list, sizeof (file_entry) *
-			      (list->size + RESIZE_STEPS));
+    if (next_free == list->size) {
+	list->list =
+	    g_realloc (list->list,
+		       sizeof (file_entry) * (list->size + RESIZE_STEPS));
 	if (!list->list)
 	    return -1;
 	list->size += RESIZE_STEPS;
@@ -462,12 +426,14 @@ handle_dirent (dir_list *list, char *filter, struct dirent *dp,
    If you change handle_path then check also handle_dirent. */
 /* Return values: -1 = failure, 0 = don't add, 1 = add to the list */
 int
-handle_path (dir_list *list, char *path,
+handle_path (dir_list *list, const char *path,
 	     struct stat *buf1, int next_free, int *link_to_dir,
-	     int *stalled_link)
+	     int *stale_link)
 {
     if (path [0] == '.' && path [1] == 0)
 	return 0;
+    if (path [0] == '.' && path [1] == '.' && path [2] == 0)
+	return 0;
     if (mc_lstat (path, buf1) == -1)
         return 0;
 
@@ -476,13 +442,13 @@ handle_path (dir_list *list, char *path,
 
     /* A link to a file or a directory? */
     *link_to_dir = 0;
-    *stalled_link = 0;
+    *stale_link = 0;
     if (S_ISLNK(buf1->st_mode)){
 	struct stat buf2;
 	if (!mc_stat (path, &buf2))
 	    *link_to_dir = S_ISDIR(buf2.st_mode) != 0;
 	else
-	    *stalled_link = 1;
+	    *stale_link = 1;
     }
 
     /* Need to grow the *list? */
@@ -497,52 +463,51 @@ handle_path (dir_list *list, char *path,
 }
 
 int
-do_load_dir (dir_list *list, sortfn *sort, int reverse, int case_sensitive, char *filter)
+do_load_dir (const char *path, dir_list *list, sortfn *sort, int reverse,
+	     int case_sensitive, const char *filter)
 {
-    DIR           *dirp;
+    DIR *dirp;
     struct dirent *dp;
-    int           status, link_to_dir, stalled_link;
-    int           next_free = 0;
-    struct stat   buf;
-    int dotdot_found = 0;
+    int status, link_to_dir, stale_link;
+    int next_free = 0;
+    struct stat st;
 
-    tree_store_start_check_cwd ();
-
-    dirp = mc_opendir (".");
-    if (!dirp){
-	tree_store_end_check ();
+    dirp = mc_opendir (path);
+    if (!dirp) {
+	message (1, MSG_ERROR, _("Cannot read directory contents"));
 	return set_zero_dir (list);
     }
-    for (dp = mc_readdir (dirp); dp; dp = mc_readdir (dirp)){
-	status = handle_dirent (list, filter, dp, &buf, next_free, &link_to_dir,
-	    &stalled_link);
+    tree_store_start_check (path);
+    while ((dp = mc_readdir (dirp))) {
+	status =
+	    handle_dirent (list, filter, dp, &st, next_free, &link_to_dir,
+			   &stale_link);
 	if (status == 0)
 	    continue;
-	if (status == -1){
+	if (status == -1) {
 	    tree_store_end_check ();
+	    mc_closedir (dirp);
 	    return next_free;
 	}
-	list->list [next_free].fnamelen = NLENGTH (dp);
-	list->list [next_free].fname = g_strdup (dp->d_name);
-	list->list [next_free].f.marked = 0;
-	list->list [next_free].f.link_to_dir = link_to_dir;
-	list->list [next_free].f.stalled_link = stalled_link;
-        list->list [next_free].f.dir_size_computed = 0;
-	list->list [next_free].buf = buf;
-	if (strcmp (dp->d_name, ".." ) == 0)
-	    dotdot_found = 1;
+	list->list[next_free].fnamelen = NLENGTH (dp);
+	list->list[next_free].fname = g_strdup (dp->d_name);
+	list->list[next_free].f.marked = 0;
+	list->list[next_free].f.link_to_dir = link_to_dir;
+	list->list[next_free].f.stale_link = stale_link;
+	list->list[next_free].f.dir_size_computed = 0;
+	list->list[next_free].st = st;
 	next_free++;
 	if (!(next_free % 32))
 	    rotate_dash ();
     }
 
     if (next_free) {
-	if (!dotdot_found)
+	/* Add ".." except the root directory */
+	if (strcmp (path, "/") != 0)
 	    add_dotdot_to_list (list, next_free++);
-	do_sort (list, sort, next_free-1, reverse, case_sensitive);
+	do_sort (list, sort, next_free - 1, reverse, case_sensitive);
     } else {
-	tree_store_end_check ();
-	return set_zero_dir (list);
+	next_free = set_zero_dir (list);
     }
 
     mc_closedir (dirp);
@@ -551,7 +516,7 @@ do_load_dir (dir_list *list, sortfn *sort, int reverse, int case_sensitive, char
 }
 
 int
-link_isdir (file_entry *file)
+link_isdir (const file_entry *file)
 {
     if (file->f.link_to_dir)
 	return 1;
@@ -560,12 +525,11 @@ link_isdir (file_entry *file)
 }
 
 int
-if_link_is_exe (char *full_name, file_entry *file)
+if_link_is_exe (const char *full_name, const file_entry *file)
 {
     struct stat b;
 
-    if (S_ISLNK (file->buf.st_mode)) {
-	mc_stat (full_name, &b);
+    if (S_ISLNK (file->st.st_mode) && !mc_stat (full_name, &b)) {
 	return is_exe (b.st_mode);
     } else
 	return 1;
@@ -582,8 +546,7 @@ alloc_dir_copy (int size)
 	if (dir_copy.list){
 
 	    for (i = 0; i < dir_copy.size; i++) {
-		if (dir_copy.list [i].fname)
-		    g_free (dir_copy.list [i].fname);
+		g_free (dir_copy.list [i].fname);
 	    }
 	    g_free (dir_copy.list);
 	    dir_copy.list = 0;
@@ -599,115 +562,100 @@ alloc_dir_copy (int size)
 
 /* If filter is null, then it is a match */
 int
-do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
-	       int case_sensitive, char *filter)
+do_reload_dir (const char *path, dir_list *list, sortfn *sort, int count,
+	       int rev, int case_sensitive, const char *filter)
 {
-    DIR           *dirp;
+    DIR *dirp;
     struct dirent *dp;
-    int           next_free = 0;
-    int           i, found, status, link_to_dir, stalled_link;
-    struct stat   buf;
-    int		  tmp_len;  /* For optimisation */
-    int 	  dotdot_found = 0;
+    int next_free = 0;
+    int i, status, link_to_dir, stale_link;
+    struct stat st;
+    int marked_cnt;
+    GHashTable *marked_files;
 
-    tree_store_start_check_cwd ();
-    dirp = mc_opendir (".");
+    dirp = mc_opendir (path);
     if (!dirp) {
- 	clean_dir (list, count);
-	tree_store_end_check ();
+	message (1, MSG_ERROR, _("Cannot read directory contents"));
+	clean_dir (list, count);
 	return set_zero_dir (list);
     }
 
+    tree_store_start_check (path);
+    marked_files = g_hash_table_new (g_str_hash, g_str_equal);
     alloc_dir_copy (list->size);
-    for (i = 0; i < count; i++){
-	dir_copy.list [i].fnamelen = list->list [i].fnamelen;
-	dir_copy.list [i].fname =    list->list [i].fname;
-	dir_copy.list [i].f.marked = list->list [i].f.marked;
-        dir_copy.list [i].f.dir_size_computed = list->list [i].f.dir_size_computed;
-	dir_copy.list [i].f.link_to_dir = list->list [i].f.link_to_dir;
-	dir_copy.list [i].f.stalled_link = list->list [i].f.stalled_link;
+    for (marked_cnt = i = 0; i < count; i++) {
+	dir_copy.list[i].fnamelen = list->list[i].fnamelen;
+	dir_copy.list[i].fname = list->list[i].fname;
+	dir_copy.list[i].f.marked = list->list[i].f.marked;
+	dir_copy.list[i].f.dir_size_computed =
+	    list->list[i].f.dir_size_computed;
+	dir_copy.list[i].f.link_to_dir = list->list[i].f.link_to_dir;
+	dir_copy.list[i].f.stale_link = list->list[i].f.stale_link;
+	if (list->list[i].f.marked) {
+	    g_hash_table_insert (marked_files, dir_copy.list[i].fname,
+				 &dir_copy.list[i]);
+	    marked_cnt++;
+	}
     }
 
-    for (dp = mc_readdir (dirp); dp; dp = mc_readdir (dirp)){
-	status = handle_dirent (list, filter, dp, &buf, next_free, &link_to_dir,
-	    &stalled_link);
+    while ((dp = mc_readdir (dirp))) {
+	status =
+	    handle_dirent (list, filter, dp, &st, next_free, &link_to_dir,
+			   &stale_link);
 	if (status == 0)
 	    continue;
 	if (status == -1) {
 	    mc_closedir (dirp);
 	    /* Norbert (Feb 12, 1997):
-	     Just in case someone finds this memory leak:
-	     -1 means big trouble (at the moment no memory left),
-	     I don't bother with further cleanup because if one gets to
-	     this point he will have more problems than a few memory
-	     leaks and because one 'clean_dir' would not be enough (and
-	     because I don't want to spent the time to make it working,
-             IMHO it's not worthwhile).
-	    clean_dir (&dir_copy, count);
-             */
+	       Just in case someone finds this memory leak:
+	       -1 means big trouble (at the moment no memory left),
+	       I don't bother with further cleanup because if one gets to
+	       this point he will have more problems than a few memory
+	       leaks and because one 'clean_dir' would not be enough (and
+	       because I don't want to spent the time to make it working,
+	       IMHO it's not worthwhile).
+	       clean_dir (&dir_copy, count);
+	     */
 	    tree_store_end_check ();
+	    g_hash_table_destroy (marked_files);
 	    return next_free;
 	}
 
-	tmp_len = NLENGTH (dp);
-	for (found = i = 0; i < count; i++)
-	    if (tmp_len == dir_copy.list [i].fnamelen
-		&& !strcmp (dp->d_name, dir_copy.list [i].fname)){
-		list->list [next_free].f.marked = dir_copy.list [i].f.marked;
-                found = 1;
-		break;
+	list->list[next_free].f.marked = 0;
+
+	/*
+	 * If we have marked files in the copy, scan through the copy
+	 * to find matching file.  Decrease number of remaining marks if
+	 * we copied one.
+	 */
+	if (marked_cnt > 0) {
+	    if ((g_hash_table_lookup (marked_files, dp->d_name))) {
+		list->list[next_free].f.marked = 1;
+		marked_cnt--;
 	    }
+	}
 
-	if (!found)
-	    list->list [next_free].f.marked = 0;
-
-	list->list [next_free].fnamelen = tmp_len;
-	list->list [next_free].fname = g_strdup (dp->d_name);
-	list->list [next_free].f.link_to_dir = link_to_dir;
-	list->list [next_free].f.stalled_link = stalled_link;
-        list->list [next_free].f.dir_size_computed = 0;
-	list->list [next_free].buf = buf;
-	if (strcmp (dp->d_name, ".." ) == 0)
-	    dotdot_found = 1;
+	list->list[next_free].fnamelen = NLENGTH (dp);
+	list->list[next_free].fname = g_strdup (dp->d_name);
+	list->list[next_free].f.link_to_dir = link_to_dir;
+	list->list[next_free].f.stale_link = stale_link;
+	list->list[next_free].f.dir_size_computed = 0;
+	list->list[next_free].st = st;
 	next_free++;
 	if (!(next_free % 16))
 	    rotate_dash ();
     }
     mc_closedir (dirp);
     tree_store_end_check ();
+    g_hash_table_destroy (marked_files);
     if (next_free) {
-	if (!dotdot_found)
+	/* Add ".." except the root directory */
+	if (strcmp (path, "/") != 0)
 	    add_dotdot_to_list (list, next_free++);
-	do_sort (list, sort, next_free-1, rev, case_sensitive);
-    }
-    else
+	do_sort (list, sort, next_free - 1, rev, case_sensitive);
+    } else
 	next_free = set_zero_dir (list);
     clean_dir (&dir_copy, count);
     return next_free;
 }
 
-char *
-sort_type_to_name (sortfn *sort_fn)
-{
-    int i;
-
-    for (i = 0; i < SORT_TYPES; i++)
-	if ((sortfn *) (sort_orders [i].sort_fn) == sort_fn)
-	    return _(sort_orders [i].sort_name);
-
-    return _("Unknown");
-}
-
-sortfn *
-sort_name_to_type (char *sname)
-{
-    int i;
-
-    for (i = 0; i < SORT_TYPES; i++)
-	if ( g_strcasecmp (sort_orders [i].sort_name, sname) == 0)
-	    return (sortfn *) sort_orders [i].sort_fn;
-
-    /* default case */
-    return (sortfn *) sort_name;
-}
-
