@@ -14,58 +14,56 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
 #include <stdio.h>
-#include "tty.h"
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/param.h>
+
 #include "global.h"
+#include "tty.h"
 #include "mouse.h"		/* Gpm_Event */
 #include "color.h"
-#include "dlg.h"
+#include "dialog.h"
 #include "info.h"
 #include "dir.h"		/* required by panel */
 #include "panel.h"		/* for the panel structure */
-#include "main.h"		/* opanel, cpanel definitions */
-#include "win.h"		/* print_bytesize */
+#include "main.h"		/* other_panel, current_panel definitions */
+#include "util.h"		/* size_trunc_len */
 #include "layout.h"
 #include "key.h"		/* is_idle() */
-#include "x.h"
 #include "mountlist.h"
 
 #ifndef VERSION
 #   define VERSION "undefined"
 #endif
 
+struct WInfo {
+    Widget widget;
+    int ready;
+};
+
 /* Have we called the init_my_statfs routine? */
 static int initialized;
 static struct my_statfs myfs_stats;
 
-static int info_event (Gpm_Event *event, WInfo *info)
-{
-    return 0;
-}
-
-static void info_box (Dlg_head *h, WInfo *info)
+static void info_box (Dlg_head *h, struct WInfo *info)
 {
     standend ();
-    if (hascolors)
-	attrset (NORMAL_COLOR);
+    attrset (NORMAL_COLOR);
     widget_erase (&info->widget);
     draw_double_box (h, info->widget.y,  info->widget.x,
 	             info->widget.lines, info->widget.cols);
 }
 
 static void
-info_show_info (WInfo *info)
+info_show_info (struct WInfo *info)
 {
     static int i18n_adjust=0;
-    static char *file_label;
+    static const char *file_label;
     
-    struct stat buf;
+    struct stat st;
 
     if (!is_idle ())
 	return;
@@ -73,34 +71,26 @@ info_show_info (WInfo *info)
     info_box (info->widget.parent, info);
     attrset (MARKED_COLOR);
     widget_move (&info->widget, 1, 3);
-    printw (_("Midnight Commander %s"), VERSION);
+    printw (const_cast(char *, _("Midnight Commander %s")), VERSION);
     attrset (NORMAL_COLOR);
     widget_move (&info->widget, 2, 1);
-    /* .ado: info->widget.x has wrong value (==0) on NT and OS/2, why? */
-#ifndef OS2_NT
-    hline (ACS_HLINE|NORMAL_COLOR, info->widget.x-2);
-#endif
+    hline (ACS_HLINE|NORMAL_COLOR, info->widget.cols-2);
     if (get_current_type () != view_listing)
 	return;
 
     if (!info->ready)
 	return;
     
-    my_statfs (&myfs_stats, cpanel->cwd);
-    buf = cpanel->dir.list [cpanel->selected].buf;
-#ifdef OS2_NT
-    /* .ado: for OS/2 and NT, st_dev must > 0 */
-    if ((signed char) buf.st_dev < 0)
-	    return;
-#endif
-    
+    my_statfs (&myfs_stats, current_panel->cwd);
+    st = current_panel->dir.list [current_panel->selected].st;
+
     /* Print only lines which fit */
     
     if(!i18n_adjust) {
+	/* This printf pattern string is used as a reference for size */
 	file_label=_("File:       %s");
 	i18n_adjust=strlen(file_label)+2;
     }
-    /*The printf pattern string is used as a reference for size*/
     
     switch (info->widget.lines-2){
 	/* Note: all cases are fall-throughs */
@@ -110,7 +100,7 @@ info_show_info (WInfo *info)
     case 16:
 	widget_move (&info->widget, 16, 3);
 	if (myfs_stats.nfree >0 || myfs_stats.nodes > 0)
-	    printw (_("Free nodes %d (%d%%) of %d"),
+	    printw (const_cast(char *, _("Free nodes: %d (%d%%) of %d")),
 		    myfs_stats.nfree,
 		    myfs_stats.total
 		    ? 100 * myfs_stats.nfree / myfs_stats.nodes : 0,
@@ -121,86 +111,89 @@ info_show_info (WInfo *info)
     case 15:
 	widget_move (&info->widget, 15, 3);
 	if (myfs_stats.avail > 0 || myfs_stats.total > 0){
-	    addstr (_("Free space "));
-	    print_bytesize (myfs_stats.avail, 1);
-	    printw (_(" (%d%%) of "), myfs_stats.total
-		    ? 100 * myfs_stats.avail / myfs_stats.total : 0);
-	    print_bytesize (myfs_stats.total, 1);
+	    char buffer1 [6], buffer2[6];
+	    size_trunc_len (buffer1, 5, myfs_stats.avail, 1);
+	    size_trunc_len (buffer2, 5, myfs_stats.total, 1);
+	    printw (const_cast(char *, _("Free space: %s (%d%%) of %s")), buffer1, myfs_stats.total ?
+		    (int)(100 * (double)myfs_stats.avail / myfs_stats.total) : 0,
+		    buffer2);
 	} else
 	    addstr (_("No space information"));
 
     case 14:
 	widget_move (&info->widget, 14, 3);
-	printw (_("Type:      %s "), myfs_stats.typename ? myfs_stats.typename : _("non-local vfs"));
-	if (myfs_stats.type != 0xffff && myfs_stats.type != 0xffffffff)
+	printw (const_cast(char *, _("Type:      %s ")), myfs_stats.typename ? myfs_stats.typename : _("non-local vfs"));
+	if (myfs_stats.type != 0xffff && myfs_stats.type != -1)
 	    printw (" (%Xh)", myfs_stats.type);
 
     case 13:
 	widget_move (&info->widget, 13, 3);
-	printw (_("Device:    %s"),
+	printw (const_cast(char *, _("Device:    %s")),
 		name_trunc (myfs_stats.device, info->widget.cols - i18n_adjust));
     case 12:
 	widget_move (&info->widget, 12, 3);
-	printw (_("Filesystem: %s"),
+	printw (const_cast(char *, _("Filesystem: %s")),
 		name_trunc (myfs_stats.mpoint, info->widget.cols - i18n_adjust));
 
     case 11:
 	widget_move (&info->widget, 11, 3);
-	printw (_("Accessed:  %s"), file_date (buf.st_atime));
+	printw (const_cast(char *, _("Accessed:  %s")), file_date (st.st_atime));
 	
     case 10:
 	widget_move (&info->widget, 10, 3);
-	printw (_("Modified:  %s"), file_date (buf.st_mtime));
+	printw (const_cast(char *, _("Modified:  %s")), file_date (st.st_mtime));
 	
     case 9:
 	widget_move (&info->widget, 9, 3);
-	printw (_("Created:   %s"), file_date (buf.st_ctime));
+	printw (const_cast(char *, _("Created:   %s")), file_date (st.st_ctime));
 
     case 8:
 	widget_move (&info->widget, 8, 3);
 #if 0
-#ifdef HAVE_ST_RDEV
-	if (buf.st_rdev)
+#ifdef HAVE_STRUCT_STAT_ST_RDEV
+	if (st.st_rdev)
 	    printw ("Inode dev: major: %d, minor: %d",
-		    buf.st_rdev >> 8, buf.st_rdev & 0xff);
+		    st.st_rdev >> 8, st.st_rdev & 0xff);
 	else
 #endif
 #endif
 	{
-	    printw (_("Size:      "));
-	    print_bytesize (buf.st_size, 0);
-#ifdef HAVE_ST_BLOCKS
-	    printw (_(" (%d blocks)"), buf.st_blocks);
+	    char buffer[10];
+	    size_trunc_len(buffer, 9, st.st_size, 0);
+	    printw (const_cast(char *, _("Size:      %s")), buffer);
+#ifdef HAVE_STRUCT_STAT_ST_BLOCKS
+	    printw (const_cast(char *, (st.st_blocks==1) ?
+		_(" (%ld block)") : _(" (%ld blocks)")), (long) st.st_blocks);
 #endif
 	}
 	
     case 7:
 	widget_move (&info->widget, 7, 3);
-	printw (_("Owner:     %s/%s"), get_owner (buf.st_uid),
-		get_group (buf.st_gid));
+	printw (const_cast(char *, _("Owner:     %s/%s")), get_owner (st.st_uid),
+		get_group (st.st_gid));
 	
     case 6:
 	widget_move (&info->widget, 6, 3);
-	printw (_("Links:     %d"), buf.st_nlink);
+	printw (const_cast(char *, _("Links:     %d")), (int) st.st_nlink);
 	
     case 5:
 	widget_move (&info->widget, 5, 3);
-	printw (_("Mode:      %s (%04o)"),
-		string_perm (buf.st_mode), buf.st_mode & 07777);
+	printw (const_cast(char *, _("Mode:      %s (%04o)")),
+		string_perm (st.st_mode), st.st_mode & 07777);
 	
     case 4:
 	widget_move (&info->widget, 4, 3);
-	printw (_("Location:  %Xh:%Xh"), (int)buf.st_dev, (int)buf.st_ino);
+	printw (const_cast(char *, _("Location:  %Xh:%Xh")), (int)st.st_dev, (int)st.st_ino);
 	
     case 3:
 	widget_move (&info->widget, 3, 2);
-	/* .ado: fname is invalid if selected == 0 && ifno called from current panel */
-	if (cpanel->selected){
-		printw (file_label,
-			name_trunc (cpanel->dir.list [cpanel->selected].fname,
+	/* .ado: fname is invalid if selected == 0 && info called from current panel */
+	if (current_panel->selected){
+		printw (const_cast(char *, file_label),
+			name_trunc (current_panel->dir.list [current_panel->selected].fname,
 				    info->widget.cols - i18n_adjust));
 	} else
-		printw (_("File:       None"));
+		addstr (_("File:       None"));
      
     case 2:
     case 1:
@@ -211,7 +204,7 @@ info_show_info (WInfo *info)
 
 static void info_hook (void *data)
 {
-    WInfo *info = (WInfo *) data;
+    struct WInfo *info = (struct WInfo *) data;
     Widget *other_widget;
     
     other_widget = get_panel_widget (get_current_index ());
@@ -224,38 +217,39 @@ static void info_hook (void *data)
     info_show_info (info);
 }
 
-static void info_destroy (WInfo *info)
+static cb_ret_t
+info_callback (struct WInfo *info, widget_msg_t msg, int parm)
 {
-    delete_hook (&select_file_hook, info_hook);
-}
-
-static int info_callback (Dlg_head *h, WInfo *info, int msg, int par)
-{
-    switch (msg){
+    switch (msg) {
 
     case WIDGET_INIT:
 	add_hook (&select_file_hook, info_hook, info);
 	info->ready = 0;
-	break;
+	return MSG_HANDLED;
 
     case WIDGET_DRAW:
 	info_hook (info);
 	info_show_info (info);
-	return 1;
+	return MSG_HANDLED;
 
     case WIDGET_FOCUS:
-	return 0;
+	return MSG_NOT_HANDLED;
+
+    case WIDGET_DESTROY:
+	delete_hook (&select_file_hook, info_hook);
+	return MSG_HANDLED;
+
+    default:
+	return default_proc (msg, parm);
     }
-    return default_proc (h, msg, par);
 }
 			   
-WInfo *info_new ()
+struct WInfo *info_new ()
 {
-    WInfo *info = g_new (WInfo, 1);
+    struct WInfo *info = g_new (struct WInfo, 1);
 
     init_widget (&info->widget, 0, 0, 0, 0, (callback_fn)
-		 info_callback, (destroy_fn) info_destroy,
-		 (mouse_h) info_event, NULL);
+		 info_callback, NULL);
 
     /* We do not want the cursor */
     widget_want_cursor (info->widget, 0);
