@@ -1,13 +1,12 @@
 #ifndef __KEY_H
 #define __KEY_H
 
-#include "mouse.h" /* Gpm_Event */
-
 void init_key (void);
 void init_key_input_fd (void);
 void done_key (void);
 
-int get_event (Gpm_Event *event, int redo_event, int block);
+struct Gpm_Event;
+int get_event (struct Gpm_Event *event, int redo_event, int block);
 int is_idle (void);
 
 int mi_getch (void);
@@ -57,7 +56,7 @@ int is_quit_char (int c);
 /* Return code for the mouse sequence */
 #define MCKEY_MOUSE     -2
 
-int define_sequence (int code, char *seq, int action);
+int define_sequence (int code, const char *seq, int action);
 
 /* internally used in key.c, defined in keyxtra.c */
 void load_xtra_key_defines (void);
@@ -70,12 +69,11 @@ int get_key_code (int nodelay);
 
 typedef const struct {
     int code;
-    char *name;
-    char *longname;
+    const char *name;
+    const char *longname;
 } key_code_name_t;
 
 extern key_code_name_t key_name_conv_tab [];
-extern int we_are_background;
 
 /* Set keypad mode (xterm and linux console only) */
 void numeric_keypad_mode (void);
