@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* }}} */
 
@@ -38,6 +38,8 @@
  *
  */
 #include <config.h>
+
+#include "global.h"
 #include "mouse.h" /* required before key.h */
 #include "key.h"
 #include "tty.h"
@@ -280,7 +282,7 @@
 #ifdef __USE_QNX_TI
     
 /* OS/implementation specific key-define struct */
-typedef struct qnx_key_define_s {
+typedef const struct qnx_key_define_s {
     int mc_code;
     int str_idx;
 } qnx_key_define_t;
