@@ -16,7 +16,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #include <config.h>
@@ -45,10 +45,9 @@
 #include "win.h"
 #include "color.h"
 #include "key.h"
-#include "dlg.h"
+#include "dialog.h"
 #include "widget.h"
 #include "command.h"
-#include "dialog.h"		/* For do_refresh() */
 #include "profile.h"		/* For sync_profiles() */
 #include "mouse.h"
 #define WANT_WIDGETS
@@ -72,7 +71,7 @@
 int nice_rotating_dash = 1;
 
 /* If set, then we have to call the layout_change routine from main */
-int layout_do_change = 0;
+static int layout_do_change = 0;
 
 /* Set if the panels are split horizontally */
 int horizontal_split = 0;
@@ -92,7 +91,7 @@ int output_lines = 0;
 /* Set if the command prompt is to be displayed */
 int command_prompt = 1;
 
-/* Set if the nice and usefull keybar is visible */
+/* Set if the nice and useful keybar is visible */
 int keybar_visible = 1;
 
 /* Set if the nice message (hint) bar is visible */
@@ -142,13 +141,13 @@ static int height;
 #define BY      12
 
 #define B_2LEFT B_USER
-#define B_2RIGHT B_USER + 1
-#define B_PLUS B_USER + 2
-#define B_MINUS B_USER + 3
+#define B_2RIGHT (B_USER + 1)
+#define B_PLUS (B_USER + 2)
+#define B_MINUS (B_USER + 3)
 
 static Dlg_head *layout_dlg;
 
-static char *s_split_direction [2] = {
+static const char *s_split_direction [2] = {
     N_("&Vertical"), 
     N_("&Horizontal")
 };
@@ -156,25 +155,24 @@ static char *s_split_direction [2] = {
 static WRadio *radio_widget;
 
 static struct {
-    char   *text;
+    const char   *text;
     int    *variable;
     WCheck *widget;
-    char   *tkname;
 } check_options [] = {
-    { N_("&Xterm window title"), &xterm_title,   0, "t" },
-    { N_("h&Intbar visible"),  &message_visible, 0, "v" },
-    { N_("&Keybar visible"),   &keybar_visible,  0, "k" },
-    { N_("command &Prompt"),   &command_prompt,  0, "p" },
-    { N_("show &Mini status"), &show_mini_info,  0, "m" },
-    { N_("menu&Bar visible"),  &menubar_visible, 0, "me" },
-    { N_("&Equal split"),      &equal_split,     0, "eq" },
-    { N_("pe&Rmissions"),      &permission_mode, 0, "pr" },
-    { N_("&File types"),       &filetype_mode,   0, "ft" },
-    { 0, 0, 0, 0 }
+    { N_("&Xterm window title"), &xterm_title,   0 },
+    { N_("h&Intbar visible"),  &message_visible, 0 },
+    { N_("&Keybar visible"),   &keybar_visible,  0 },
+    { N_("command &Prompt"),   &command_prompt,  0 },
+    { N_("show &Mini status"), &show_mini_info,  0 },
+    { N_("menu&Bar visible"),  &menubar_visible, 0 },
+    { N_("&Equal split"),      &equal_split,     0 },
+    { N_("pe&Rmissions"),      &permission_mode, 0 },
+    { N_("&File types"),       &filetype_mode,   0 },
+    { 0, 0, 0 }
 };
 
 static int first_width, second_width;
-static char *title1, *title2, *title3, *output_lines_label;
+static const char *output_lines_label;
 
 static WButton *bleft_widget, *bright_widget;
 
@@ -225,7 +223,7 @@ static void update_split (void)
 	printw ("%03d", COLS - _first_panel_size);
 }
 
-static int b2left_cback (int action, void *data)
+static int b2left_cback (int action)
 {
     if (_equal_split){
 	/* Turn equal split off */
@@ -238,7 +236,7 @@ static int b2left_cback (int action, void *data)
     return 0;
 }
 
-static int b2right_cback (int action, void *data)
+static int b2right_cback (int action)
 {
     if (_equal_split){
 	/* Turn equal split off */
@@ -251,23 +249,24 @@ static int b2right_cback (int action, void *data)
     return 0;
 }
 
-static int bplus_cback (int action, void *data)
+static int bplus_cback (int action)
 {
     if (_output_lines < 99)
 	_output_lines++;
     return 0;
 }
 
-static int bminus_cback (int action, void *data)
+static int bminus_cback (int action)
 {
     if (_output_lines > 0)
 	_output_lines--;
     return 0;
 }
 
-static int layout_callback (struct Dlg_head *h, int Id, int Msg)
+static cb_ret_t
+layout_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    switch (Msg){
+    switch (msg) {
     case DLG_DRAW:
     	/*When repainting the whole dialog (e.g. with C-l) we have to
     	  update everything*/
@@ -277,18 +276,7 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
     	old_horizontal_split = -1;
     	old_output_lines     = -1;
 
-	attrset (COLOR_NORMAL);
-	draw_box (h, 2, 4, 6, first_width);
-	draw_box (h, 8, 4, 4, first_width);
-	draw_box (h, 2, 5 + first_width, 10, second_width);
-
 	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 2, 5);
-	addstr (title1);
-	dlg_move (h, 8, 5);
-	addstr (title2);
-	dlg_move (h, 2, 6 + first_width);
-	addstr (title3);
 	update_split ();
 	dlg_move (h, 6, 13);
 	addch ('=');
@@ -302,7 +290,7 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 		printw ("%02d", _output_lines);
 	    }
 	}
-	break;
+	return MSG_HANDLED;
 
     case DLG_POST_KEY:
 	_filetype_mode = check_options [8].widget->state & C_BOOL;
@@ -344,124 +332,130 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 		printw ("%02d", _output_lines);
 	    }
 	}
-	break;
+	return MSG_HANDLED;
 
-    case DLG_END:
-	break;
+    default:
+	return default_dlg_callback (h, msg, parm);
     }
-    return 0;
 }
 
-static void init_layout (void)
+static void
+init_layout (void)
 {
     static int i18n_layt_flag = 0;
-	static int b1, b2, b3;
-    int i = sizeof (s_split_direction) / sizeof(char*) ;
-	char* ok_button = _("&OK");
-	char* cancel_button = _("&Cancel");
-	char* save_button = _("&Save");
-    
-    if (!i18n_layt_flag)
-    {
-		register int l1;
-		
-		first_width = 19; /* length of line with '<' '>' buttons */
-		
-		title1 = _(" Panel split ");
-		title2 = _(" Highlight... ");
-		title3 = _(" Other options ");
-		output_lines_label = _("output lines");
-		
-		while (i--)
-		{
-		    s_split_direction [i] = _(s_split_direction [i]);
-			l1 = strlen (s_split_direction [i]) + 7;
-			if (l1 > first_width)
-				first_width = l1;
-		}
+    static int b1, b2, b3;
+    int i = sizeof (s_split_direction) / sizeof (char *);
+    const char *ok_button = _("&OK");
+    const char *cancel_button = _("&Cancel");
+    const char *save_button = _("&Save");
+    static const char *title1, *title2, *title3;
 
-		for (i = 0; i <= 8; i++)
-		{
-			check_options[i].text = _(check_options[i].text);
-			l1 = strlen (check_options[i].text) + 7;
-			if (l1 > first_width)
-				first_width = l1;
-		}
+    if (!i18n_layt_flag) {
+	register int l1;
 
-		l1 = strlen (title1) + 1;
-		if (l1 > first_width)
-			first_width = l1;
-		
-		l1 = strlen (title2) + 1;
-		if (l1 > first_width)
-			first_width = l1;
-		
+	first_width = 19;	/* length of line with '<' '>' buttons */
 
-		second_width = strlen (title3) + 1;
-		for (i = 0; i < 6; i++)
-		{
-			check_options[i].text = _(check_options[i].text);
-			l1 = strlen (check_options[i].text) + 7;
-			if (l1 > second_width)
-				second_width = l1;
-		}
-		if (console_flag)
-		{
-			l1 = strlen (output_lines_label) + 13;
-			if (l1 > second_width)
-				second_width = l1;
-		}
+	title1 = _(" Panel split ");
+	title2 = _(" Highlight... ");
+	title3 = _(" Other options ");
+	output_lines_label = _("output lines");
 
-		/* 
-		 * alex@bcs.zp.ua:
-		 * To be completely correct, one need to check if the title
-		 * does not exceed dialog length and total length of 3 buttons
-		 * allows their placement in one row. But assuming this dialog
-		 * is wide enough, I don't include such a tests.
-		 *
-		 * Now the last thing to do - properly space buttons...
-		 */
-		l1 = 11 + strlen (ok_button)   /* 14 - all brackets and inner space */
-		 	+ strlen (save_button)     /* notice: it is 3 char less because */
-			+ strlen (cancel_button);  /* of '&' char in button text */
-		
-		i = (first_width + second_width - l1) / 4;
-		b1 = 5 + i;
-		b2 = b1 + strlen(ok_button) + i + 6;
-		b3 = b2 + strlen(save_button) + i + 4;
+	while (i--) {
+	    s_split_direction[i] = _(s_split_direction[i]);
+	    l1 = strlen (s_split_direction[i]) + 7;
+	    if (l1 > first_width)
+		first_width = l1;
+	}
 
-		i18n_layt_flag = 1;
+	for (i = 0; i <= 8; i++) {
+	    check_options[i].text = _(check_options[i].text);
+	    l1 = strlen (check_options[i].text) + 7;
+	    if (l1 > first_width)
+		first_width = l1;
+	}
+
+	l1 = strlen (title1) + 1;
+	if (l1 > first_width)
+	    first_width = l1;
+
+	l1 = strlen (title2) + 1;
+	if (l1 > first_width)
+	    first_width = l1;
+
+
+	second_width = strlen (title3) + 1;
+	for (i = 0; i < 6; i++) {
+	    check_options[i].text = _(check_options[i].text);
+	    l1 = strlen (check_options[i].text) + 7;
+	    if (l1 > second_width)
+		second_width = l1;
+	}
+	if (console_flag) {
+	    l1 = strlen (output_lines_label) + 13;
+	    if (l1 > second_width)
+		second_width = l1;
+	}
+
+	/* 
+	 * alex@bcs.zp.ua:
+	 * To be completely correct, one need to check if the title
+	 * does not exceed dialog length and total length of 3 buttons
+	 * allows their placement in one row. But assuming this dialog
+	 * is wide enough, I don't include such a tests.
+	 *
+	 * Now the last thing to do - properly space buttons...
+	 */
+	l1 = 11 + strlen (ok_button)	/* 14 - all brackets and inner space */
+	    +strlen (save_button)	/* notice: it is 3 char less because */
+	    +strlen (cancel_button);	/* of '&' char in button text */
+
+	i = (first_width + second_width - l1) / 4;
+	b1 = 5 + i;
+	b2 = b1 + strlen (ok_button) + i + 6;
+	b3 = b2 + strlen (save_button) + i + 4;
+
+	i18n_layt_flag = 1;
     }
 
-    layout_dlg = create_dlg (0, 0, 15, first_width + second_width + 9, 
-		dialog_colors, layout_callback,
-		"[Layout]", _("Layout"), DLG_CENTER);
+    layout_dlg =
+	create_dlg (0, 0, 15, first_width + second_width + 9,
+		    dialog_colors, layout_callback, "[Layout]",
+		    _("Layout"), DLG_CENTER | DLG_REVERSE);
+
+    add_widget (layout_dlg, groupbox_new (4, 2, first_width, 6, title1));
+    add_widget (layout_dlg, groupbox_new (4, 8, first_width, 4, title2));
+    add_widget (layout_dlg,
+		groupbox_new (5 + first_width, 2, second_width, 10,
+			      title3));
 
     add_widget (layout_dlg,
-		button_new (BY, b3, B_CANCEL, NORMAL_BUTTON, cancel_button, 0, 0, "c"));
+		button_new (BY, b3, B_CANCEL, NORMAL_BUTTON, cancel_button,
+			    0));
     add_widget (layout_dlg,
-		button_new (BY, b2, B_EXIT, NORMAL_BUTTON, save_button, 0, 0, "s"));
+		button_new (BY, b2, B_EXIT, NORMAL_BUTTON, save_button,
+			    0));
     add_widget (layout_dlg,
-		button_new (BY, b1, B_ENTER, DEFPUSH_BUTTON, ok_button, 0, 0, "o"));
-    if (console_flag){
+		button_new (BY, b1, B_ENTER, DEFPUSH_BUTTON, ok_button,
+			    0));
+    if (console_flag) {
 	add_widget (layout_dlg,
-		    button_new (9, 12 + first_width, B_MINUS, NARROW_BUTTON, "&-",
-			bminus_cback, 0, NULL));
+		    button_new (9, 12 + first_width, B_MINUS,
+				NARROW_BUTTON, "&-", bminus_cback));
 	add_widget (layout_dlg,
-		    button_new (9, 7 + first_width, B_PLUS, NARROW_BUTTON, "&+", 
-			bplus_cback, 0, NULL));
+		    button_new (9, 7 + first_width, B_PLUS, NARROW_BUTTON,
+				"&+", bplus_cback));
     }
+#define XTRACT(i) *check_options[i].variable, check_options[i].text
 
-#define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options[i].tkname
-
-    for (i = 0; i < 6; i++){
-	check_options [i].widget = check_new (8 - i, 7 + first_width, XTRACT(i));
-	add_widget (layout_dlg, check_options [i].widget);
+    for (i = 0; i < 6; i++) {
+	check_options[i].widget =
+	    check_new (8 - i, 7 + first_width, XTRACT (i));
+	add_widget (layout_dlg, check_options[i].widget);
     }
-    check_options [8].widget = check_new (10, 6, XTRACT(8));
-    add_widget (layout_dlg, check_options [8].widget);
-    check_options [7].widget = check_new (9, 6, XTRACT(7));
-    add_widget (layout_dlg, check_options [7].widget);
+    check_options[8].widget = check_new (10, 6, XTRACT (8));
+    add_widget (layout_dlg, check_options[8].widget);
+    check_options[7].widget = check_new (9, 6, XTRACT (7));
+    add_widget (layout_dlg, check_options[7].widget);
 
     _filetype_mode = filetype_mode;
     _permission_mode = permission_mode;
@@ -471,32 +465,35 @@ static void init_layout (void)
     _keybar_visible = keybar_visible;
     _message_visible = message_visible;
     _xterm_title = xterm_title;
-    bright_widget = button_new(6, 15, B_2RIGHT, NARROW_BUTTON, "&>", b2right_cback, 0, ">");
+    bright_widget =
+	button_new (6, 15, B_2RIGHT, NARROW_BUTTON, "&>", b2right_cback);
     add_widget (layout_dlg, bright_widget);
-    bleft_widget = button_new (6, 9, B_2LEFT, NARROW_BUTTON, "&<", b2left_cback, 0, "<");
+    bleft_widget =
+	button_new (6, 9, B_2LEFT, NARROW_BUTTON, "&<", b2left_cback);
     add_widget (layout_dlg, bleft_widget);
-    check_options [6].widget = check_new (5, 6, XTRACT(6));
+    check_options[6].widget = check_new (5, 6, XTRACT (6));
     old_first_panel_size = -1;
     old_horizontal_split = -1;
-    old_output_lines     = -1;
-    
+    old_output_lines = -1;
+
     _first_panel_size = first_panel_size;
     _output_lines = output_lines;
-    add_widget (layout_dlg, check_options [6].widget);
-    radio_widget = radio_new (3, 6, 2, s_split_direction, 1, "r");
+    add_widget (layout_dlg, check_options[6].widget);
+    radio_widget = radio_new (3, 6, 2, s_split_direction, 1);
     add_widget (layout_dlg, radio_widget);
     radio_widget->sel = horizontal_split;
 }
 
-void layout_change (void)
+static void
+layout_change (void)
 {
     setup_panels ();
     layout_do_change = 0;
     /* re-init the menu, because perhaps there was a change in the way 
-	how the panel are split (horizontal/vertical). */
-    done_menu();
-    init_menu();
-    menubar_arrange(the_menubar);
+       how the panel are split (horizontal/vertical). */
+    done_menu ();
+    init_menu ();
+    menubar_arrange (the_menubar);
 }
 
 void layout_cmd (void)
@@ -609,6 +606,15 @@ void init_curses (void)
 }
 #endif /* ! HAVE_SLANG */
 
+void
+clr_scr (void)
+{
+    standend ();
+    dlg_erase (midnight_dlg);
+    mc_refresh ();
+    doupdate ();
+}
+
 void
 done_screen ()
 {
@@ -617,6 +623,7 @@ done_screen ()
     reset_shell_mode ();
     mc_noraw_mode ();
     keypad (stdscr, FALSE);
+    done_colors ();
 }
 
 static void
@@ -728,7 +735,7 @@ static void
 low_level_change_screen_size (void)
 {
 #if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#if defined TIOCGWINSZ && !defined SCO_FLAVOR
+#if defined TIOCGWINSZ
     struct winsize winsz;
 
     winsz.ws_col = winsz.ws_row = 0;
@@ -746,7 +753,7 @@ low_level_change_screen_size (void)
 	resize_subshell ();
 #endif
     }
-#endif /* TIOCGWINSZ && !SCO_FLAVOR */
+#endif /* TIOCGWINSZ */
 #endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
 }
 
@@ -754,7 +761,7 @@ void
 change_screen_size (void)
 {
 #if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#if defined TIOCGWINSZ && !defined SCO_FLAVOR
+#if defined TIOCGWINSZ
 
 #ifndef NCURSES_VERSION
     mc_noraw_mode ();
@@ -774,8 +781,7 @@ change_screen_size (void)
     setup_panels ();
 
     /* Inform currently running dialog */
-    (*current_dlg->callback) (current_dlg, current_dlg->current->dlg_id,
-			      DLG_RESIZE);
+    (*current_dlg->callback) (current_dlg, DLG_RESIZE, 0);
 
 #ifdef RESIZABLE_MENUBAR
     menubar_arrange (the_menubar);
@@ -784,7 +790,7 @@ change_screen_size (void)
     /* Now, force the redraw */
     do_refresh ();
     touchwin (stdscr);
-#endif				/* TIOCGWINSZ && !SCO_FLAVOR */
+#endif				/* TIOCGWINSZ */
 #endif				/* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
     winch_flag = 0;
 }
@@ -799,14 +805,14 @@ void use_dash (int flag)
 	ok_to_refresh--;
 }
 
-void set_hintbar(char *str) 
+void set_hintbar(const char *str) 
 {
     label_set_text (the_hint, str);
     if (ok_to_refresh > 0)
         refresh();
 }
 
-void print_vfs_message (char *msg, ...)
+void print_vfs_message (const char *msg, ...)
 {
     va_list ap;
     char str [128];
@@ -845,8 +851,8 @@ void print_vfs_message (char *msg, ...)
 
 void rotate_dash (void)
 {
-    static char rotating_dash [] = "|/-\\";
-    static int pos = 0;
+    static const char rotating_dash [] = "|/-\\";
+    static size_t pos = 0;
 
     if (!nice_rotating_dash || (ok_to_refresh <= 0))
 	return;
@@ -854,29 +860,13 @@ void rotate_dash (void)
     if (pos >= sizeof (rotating_dash)-1)
 	pos = 0;
     move (0, COLS-1);
+    attrset (NORMAL_COLOR);
     addch (rotating_dash [pos]);
     mc_refresh ();
     pos++;
 }
 
-void remove_dash (void)
-{
-    if (!nice_rotating_dash)
-	return;
-
-    /* Currently, it's much nicer with the CPU to do this instead of
-       calling do_refresh.
-
-       I should implement a routine called invalidate_region that would
-       send a draw message only to the affected views.  But for now
-       this is fine.
-    */
-    
-    move (0, COLS-1);
-    addch (' ');
-}
-
-char *get_nth_panel_name (int num)
+const char *get_nth_panel_name (int num)
 {
     static char buffer [BUF_SMALL];
     
@@ -905,7 +895,7 @@ void set_display_type (int num, int type)
 {
     int x, y, cols, lines;
     int    the_other;		/* Index to the other panel */
-    char   *file_name = 0;	/* For Quick view */
+    const char   *file_name = NULL;	/* For Quick view */
     Widget *new_widget, *old_widget;
     WPanel  *the_other_panel;
 
@@ -1001,11 +991,11 @@ void set_display_type (int num, int type)
      * - select left panel
      * - invoke menue left/tree
      * - as long as you stay in the left panel almost everything that uses
-     *   cpanel causes segfault, e.g. C-Enter, C-x c, ...
+     *   current_panel causes segfault, e.g. C-Enter, C-x c, ...
      */
 
     if (type != view_listing)
-	if (cpanel == (WPanel *) old_widget)
+	if (current_panel == (WPanel *) old_widget)
 	    current_panel = num == 0 ? right_panel : left_panel;
 }
 
@@ -1044,13 +1034,14 @@ void swap_panels ()
 	
         panel1->searching = 0;
         panel2->searching = 0;
-        if (cpanel == panel1)
+        if (current_panel == panel1)
             current_panel = panel2;
         else
             current_panel = panel1;
-        if (midnight_dlg->current->widget == panels [0].widget)
+
+        if (dlg_widget_active (panels[0].widget))
             dlg_select_widget (midnight_dlg, (void *) panels [1].widget);
-        else if (midnight_dlg->current->widget == panels [1].widget)
+        else if (dlg_widget_active (panels[1].widget))
             dlg_select_widget (midnight_dlg, (void *) panels [0].widget);
     } else {
 	WPanel *tmp_panel;
@@ -1101,14 +1092,15 @@ int get_display_type (int index)
     return panels [index].type;
 }
 
-Widget *get_panel_widget (int index)
+struct Widget *
+get_panel_widget (int index)
 {
-    return panels [index].widget;
+    return panels[index].widget;
 }
 
 int get_current_index (void)
 {
-    if (panels [0].widget == ((Widget *) cpanel))
+    if (panels [0].widget == ((Widget *) current_panel))
 	return 0;
     else
 	return 1;
@@ -1119,10 +1111,16 @@ int get_other_index (void)
     return !get_current_index ();
 }
 
+struct WPanel *
+get_other_panel (void)
+{
+    return (struct WPanel *) get_panel_widget (get_other_index ());
+}
+
 /* Returns the view type for the current panel/view */
 int get_current_type (void)
 {
-    if (panels [0].widget == (Widget *) cpanel)
+    if (panels [0].widget == (Widget *) current_panel)
 	return panels [0].type;
     else
 	return panels [1].type;
@@ -1131,7 +1129,7 @@ int get_current_type (void)
 /* Returns the view type of the unselected panel */
 int get_other_type (void)
 {
-    if (panels [0].widget == (Widget *) cpanel)
+    if (panels [0].widget == (Widget *) current_panel)
 	return panels [1].type;
     else
 	return panels [0].type;
