@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -21,6 +21,7 @@
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 /* This header needs to be included before sys/mount.h on *BSD */
 #ifdef HAVE_SYS_PARAM_H
@@ -81,19 +82,44 @@
 #include <sys/fsys.h>
 #endif
 
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+
+#ifdef HAVE_SYS_VFS_H
+#include <sys/vfs.h>
+#endif
+
+#ifdef HAVE_SYS_FILSYS_H
+#include <sys/filsys.h>		/* SVR2.  */
+#endif
+
+#ifdef HAVE_DUSTAT_H		/* AIX PS/2.  */
+#include <sys/dustat.h>
+#endif
+
+#ifdef HAVE_SYS_STATVFS_H	/* SVR4.  */
+#include <sys/statvfs.h>
+#endif
+
 #include "global.h"
 #include "mountlist.h"
-#include "fsusage.h"
 #include "util.h"
 
-/* void error (void);  FIXME -- needed? */
-
 #ifdef DOLPHIN
 /* So special that it's not worth putting this in autoconf.  */
 #undef MOUNTED_FREAD_FSTYP
 #define MOUNTED_GETMNTTBL
 #endif
 
+#if defined (__QNX__) && !defined(__QNXNTO__) && !defined (HAVE_INFOMOUNT_LIST)
+#    define HAVE_INFOMOUNT_QNX
+#endif
+
+#if defined(HAVE_INFOMOUNT_LIST) || defined(HAVE_INFOMOUNT_QNX)
+#    define HAVE_INFOMOUNT
+#endif
+
 /* A mount table entry. */
 struct mount_entry
 {
@@ -104,6 +130,17 @@ struct mount_entry
   struct mount_entry *me_next;
 };
 
+struct fs_usage
+{
+  long fsu_blocks;		/* Total blocks. */
+  long fsu_bfree;		/* Free blocks available to superuser. */
+  long fsu_bavail;		/* Free blocks available to non-superuser. */
+  long fsu_files;		/* Total file nodes. */
+  long fsu_ffree;		/* Free file nodes. */
+};
+
+static int get_fs_usage (char *path, struct fs_usage *fsp);
+
 #ifdef HAVE_INFOMOUNT_LIST
 
 static struct mount_entry *mount_list = NULL;
@@ -113,7 +150,7 @@ static struct mount_entry *mount_list = NULL;
    No prefix (like '0x') or suffix (like 'h') is expected to be
    part of CP. */
 
-static int xatoi (char *cp)
+static int xatoi (const char *cp)
 {
     int val;
 
@@ -205,7 +242,7 @@ read_filesystem_list (int need_fs_type, int all_fs)
     {
 	struct mntent *mnt;
 	FILE *fp;
-	char *devopt;
+	const char *devopt;
 
 	fp = setmntent (MOUNTED, "r");
 	if (fp == NULL)
@@ -276,7 +313,7 @@ read_filesystem_list (int need_fs_type, int all_fs)
 	struct fs_data fsd;
 
 	while ((val = getmnt (&offset, &fsd, sizeof (fsd), NOSTAT_MANY,
-			      (char *) 0)) > 0) {
+			      NULL)) > 0) {
 	    me = (struct mount_entry *) malloc (sizeof (struct mount_entry));
 	    me->me_devname = strdup (fsd.fd_req.devname);
 	    me->me_mountdir = strdup (fsd.fd_req.path);
@@ -551,7 +588,7 @@ init_my_statfs (void)
 }
 
 void
-my_statfs (struct my_statfs *myfs_stats, char *path)
+my_statfs (struct my_statfs *myfs_stats, const char *path)
 {
 #ifdef HAVE_INFOMOUNT_LIST
     int i, len = 0;
@@ -620,3 +657,136 @@ my_statfs (struct my_statfs *myfs_stats, char *path)
     }
 }
 
+#ifdef HAVE_INFOMOUNT
+
+/* Return the number of TOSIZE-byte blocks used by
+   BLOCKS FROMSIZE-byte blocks, rounding away from zero.
+   TOSIZE must be positive.  Return -1 if FROMSIZE is not positive.  */
+
+static long
+fs_adjust_blocks (long blocks, int fromsize, int tosize)
+{
+    if (tosize <= 0)
+	abort ();
+    if (fromsize <= 0)
+	return -1;
+
+    if (fromsize == tosize)	/* E.g., from 512 to 512.  */
+	return blocks;
+    else if (fromsize > tosize)	/* E.g., from 2048 to 512.  */
+	return blocks * (fromsize / tosize);
+    else			/* E.g., from 256 to 512.  */
+	return (blocks + (blocks < 0 ? -1 : 1)) / (tosize / fromsize);
+}
+
+#if defined(_AIX) && defined(_I386)
+/* AIX PS/2 does not supply statfs.  */
+static int aix_statfs (char *path, struct statfs *fsb)
+{
+    struct stat stats;
+    struct dustat fsd;
+
+    if (stat (path, &stats))
+	return -1;
+    if (dustat (stats.st_dev, 0, &fsd, sizeof (fsd)))
+	return -1;
+    fsb->f_type = 0;
+    fsb->f_bsize = fsd.du_bsize;
+    fsb->f_blocks = fsd.du_fsize - fsd.du_isize;
+    fsb->f_bfree = fsd.du_tfree;
+    fsb->f_bavail = fsd.du_tfree;
+    fsb->f_files = (fsd.du_isize - 2) * fsd.du_inopb;
+    fsb->f_ffree = fsd.du_tinode;
+    fsb->f_fsid.val[0] = fsd.du_site;
+    fsb->f_fsid.val[1] = fsd.du_pckno;
+    return 0;
+}
+#define statfs(path,fsb) aix_statfs(path,fsb)
+#endif				/* _AIX && _I386 */
+
+/* Fill in the fields of FSP with information about space usage for
+   the filesystem on which PATH resides.
+   Return 0 if successful, -1 if not. */
+
+static int
+get_fs_usage (char *path, struct fs_usage *fsp)
+{
+#ifdef STAT_STATFS3_OSF1
+    struct statfs fsd;
+
+    if (statfs (path, &fsd, sizeof (struct statfs)) != 0)
+	 return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_fsize, 512)
+#endif				/* STAT_STATFS3_OSF1 */
+
+#ifdef STAT_STATFS2_FS_DATA	/* Ultrix.  */
+    struct fs_data fsd;
+
+    if (statfs (path, &fsd) != 1)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), 1024, 512)
+    fsp->fsu_blocks = CONVERT_BLOCKS (fsd.fd_req.btot);
+    fsp->fsu_bfree = CONVERT_BLOCKS (fsd.fd_req.bfree);
+    fsp->fsu_bavail = CONVERT_BLOCKS (fsd.fd_req.bfreen);
+    fsp->fsu_files = fsd.fd_req.gtot;
+    fsp->fsu_ffree = fsd.fd_req.gfree;
+#endif
+
+#ifdef STAT_STATFS2_BSIZE	/* 4.3BSD, SunOS 4, HP-UX, AIX.  */
+    struct statfs fsd;
+
+    if (statfs (path, &fsd) < 0)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_bsize, 512)
+#endif
+
+#ifdef STAT_STATFS2_FSIZE	/* 4.4BSD.  */
+    struct statfs fsd;
+
+    if (statfs (path, &fsd) < 0)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_fsize, 512)
+#endif
+
+#ifdef STAT_STATFS4		/* SVR3, Dynix, Irix, AIX.  */
+    struct statfs fsd;
+
+    if (statfs (path, &fsd, sizeof fsd, 0) < 0)
+	return -1;
+    /* Empirically, the block counts on most SVR3 and SVR3-derived
+       systems seem to always be in terms of 512-byte blocks,
+       no matter what value f_bsize has.  */
+#if _AIX
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_bsize, 512)
+#else
+#define CONVERT_BLOCKS(b) (b)
+#ifndef _SEQUENT_		/* _SEQUENT_ is DYNIX/ptx.  */
+#ifndef DOLPHIN			/* DOLPHIN 3.8.alfa/7.18 has f_bavail */
+#define f_bavail f_bfree
+#endif
+#endif
+#endif
+#endif
+
+#ifdef STAT_STATVFS		/* SVR4.  */
+    struct statvfs fsd;
+
+    if (statvfs (path, &fsd) < 0)
+	return -1;
+    /* f_frsize isn't guaranteed to be supported.  */
+#define CONVERT_BLOCKS(b) \
+  fs_adjust_blocks ((b), fsd.f_frsize ? fsd.f_frsize : fsd.f_bsize, 512)
+#endif
+
+#if defined(CONVERT_BLOCKS) && !defined(STAT_STATFS2_FS_DATA) && !defined(STAT_READ_FILSYS)	/* !Ultrix && !SVR2.  */
+    fsp->fsu_blocks = CONVERT_BLOCKS (fsd.f_blocks);
+    fsp->fsu_bfree = CONVERT_BLOCKS (fsd.f_bfree);
+    fsp->fsu_bavail = CONVERT_BLOCKS (fsd.f_bavail);
+    fsp->fsu_files = fsd.f_files;
+    fsp->fsu_ffree = fsd.f_ffree;
+#endif
+
+    return 0;
+}
+
+#endif /* HAVE_INFOMOUNT */
