@@ -13,29 +13,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef __MOUNTLIST_H
 #define __MOUNTLIST_H
 
-/* A mount table entry. */
-struct mount_entry
-{
-  char *me_devname;		/* Device node pathname, including "/dev/". */
-  char *me_mountdir;		/* Mount point directory pathname. */
-  char *me_type;		/* "nfs", "4.2", etc. */
-  dev_t me_dev;			/* Device number of me_mountdir. */
-  struct mount_entry *me_next;
-};
-
-struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs);
-
 /* Filesystem status */
 struct my_statfs {
     int type;
     char *typename;
-    char *mpoint;
-    char *device;
+    const char *mpoint;
+    const char *device;
     int avail;
     int total;
     int nfree;
@@ -43,6 +31,6 @@ struct my_statfs {
 };
 
 void init_my_statfs (void);
-void my_statfs (struct my_statfs *myfs_stats, char *path);
+void my_statfs (struct my_statfs *myfs_stats, const char *path);
 
 #endif 	/* __MOUNTLIST_H	*/
