@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Events received by clients of this library have their coordinates 0 */
 /* based */
@@ -31,7 +31,7 @@
 #include "key.h"		/* define sequence */
 
 int mouse_enabled = 0;
-char *xmouse_seq;
+const char *xmouse_seq;
 
 #ifdef HAVE_LIBGPM
 void show_mouse_pointer (int x, int y)
