@@ -40,40 +40,44 @@ typedef struct Gpm_Event {
   enum Gpm_Etype type;
 } Gpm_Event;
 
-extern int gpm_fd;
+#endif /* !HAVE_LIBGPM */
 
-#endif
+/* General (i.e. both for xterm and gpm) mouse support definitions */
 
-/* General mouse support definitions */
+/* Constants returned from the mouse callback */
+enum { MOU_NORMAL, MOU_REPEAT };
 
-typedef int (*mouse_h)(Gpm_Event *, void *);
+/* Mouse callback */
+typedef int (*mouse_h) (Gpm_Event *, void *);
 
-#define NO_MOUSE 0
-#define GPM_MOUSE 1
-#define XTERM_MOUSE 2
+/* Type of mouse support */
+typedef enum {
+    MOUSE_NONE,		/* Not detected yet */
+    MOUSE_DISABLED,	/* Explicitly disabled by -d */
+    MOUSE_GPM,		/* Support using GPM on Linux */
+    MOUSE_XTERM		/* Support using xterm-style mouse reporting */
+} Mouse_Type;
+
+/* Type of the currently used mouse */
+extern Mouse_Type use_mouse_p;
+
+/* The mouse is currently: 1 - enabled, 0 - disabled */
+extern int mouse_enabled;
+
+/* String indicating that a mouse event has occured, usually "\E[M" */
+extern const char *xmouse_seq;
 
 void init_mouse (void);
-void shut_mouse (void);
+void enable_mouse (void);
+void disable_mouse (void);
 
-/* Type of mouse: NO_MOUSE, GPM_MOUSE or XTERM_MOUSE */
-extern int use_mouse_p;
-/* If use_mouse_p is XTERM_MOUSE: is mouse currently active? */
-extern int xmouse_flag;
+/* Mouse wheel events */
+#ifndef GPM_B_DOWN
+#define GPM_B_DOWN      32
+#endif
 
-int mouse_handler (Gpm_Event *gpm_event);
-int redo_mouse (Gpm_Event *event);
-
-/* Constants returned from mouse handlers */
-
-#define MOU_NORMAL    0x00
-#define MOU_REPEAT    0x01
-#define MOU_ENDLOOP   0x02
-#define MOU_LOCK      0x04
-
-#ifdef DEBUGMOUSE
-#define DEBUGM(data) fprintf data
-#else
-#define DEBUGM(data) 
+#ifndef GPM_B_UP
+#define GPM_B_UP        16
 #endif
 
 #ifdef HAVE_LIBGPM
