@@ -2,10 +2,9 @@
 #define __PANEL_H
 
 #include "dir.h"		/* dir_list */
-#include "dlg.h"		/* Widget */
+#include "dialog.h"		/* Widget */
 
 #define selection(p) (&(p->dir.list[p->selected]))
-#define other_panel get_other_panel()
 #define DEFAULT_USER_FORMAT "half type name | size | perm"
 
 #define LIST_TYPES	4
@@ -31,7 +30,6 @@ enum panel_display_enum {
 };
 
 struct format_e;
-struct hist_entry;
 
 typedef struct WPanel {
     Widget   widget;
@@ -41,7 +39,7 @@ typedef struct WPanel {
     int      active;		/* If panel is currently selected */
     char     cwd [MC_MAXPATHLEN];/* Current Working Directory */
     char     lwd [MC_MAXPATHLEN];/* Last Working Directory */
-    struct   hist_entry *dir_history;	/* directory history */
+    GList    *dir_history;	/* directory history */
     char     *hist_name;	/* directory history name for history file */
     int      count;		/* Number of files in dir structure */
     int      marked;		/* Count of marked files */
@@ -85,28 +83,21 @@ extern int show_mini_info;
 extern int panel_scroll_pages;
 extern int fast_reload;
 
-void paint_panel          (WPanel *panel);
 void panel_reload         (WPanel *panel);
-
 void panel_set_sort_order (WPanel *panel, sortfn *sort_order);
 void panel_re_sort        (WPanel *panel);
 
-void panel_update_contents (WPanel *panel);
+void update_dirty_panels (void);
 void panel_update_cols (Widget *widget, int frame_size);
 int set_panel_formats (WPanel *p);
 
-WPanel *get_current_panel (void);
-WPanel *get_other_panel (void);
-
-/* Useful macros to avoid too much typing */
-#define cpanel get_current_panel()
-#define opanel get_other_panel()
+#define other_panel get_other_panel()
 
 extern WPanel *left_panel;
 extern WPanel *right_panel;
 extern WPanel *current_panel;
 
-void try_to_select (WPanel *panel, char *name);
+void try_to_select (WPanel *panel, const char *name);
 
 void unmark_files (WPanel *panel);
 void select_item (WPanel *panel);
