@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
 #include <sys/types.h>		/* Needed to include local .h files */
@@ -41,6 +41,10 @@
 #include "cmd.h"
 #include "file.h"		/* safe_delete */
 
+#ifdef USE_VFS
+#include "../vfs/gc.h"
+#endif
+
 #ifdef HAVE_CHARSET
 #include "charsets.h"
 #endif
@@ -77,7 +81,7 @@ int startup_right_mode;
 int saving_setup;
 
 static const struct {
-    char *key;
+    const char *key;
     sortfn *sort_type;
 } sort_names [] = {
     { "name",      (sortfn *) sort_name },
@@ -92,7 +96,7 @@ static const struct {
 };
 
 static const struct {
-    char *key;
+    const char *key;
     int  list_type;
 } list_types [] = {
     { "full",  list_full  },
@@ -103,7 +107,7 @@ static const struct {
 };
 
 static const struct {
-    char *opt_name;
+    const char *opt_name;
     int  opt_type;
 } panel_types [] = {
     { "listing",   view_listing },
@@ -114,7 +118,7 @@ static const struct {
 };
 
 static const struct {
-    char *opt_name;
+    const char *opt_name;
     int *opt_addr;
 } layout [] = {
     { "equal_split", &equal_split },
@@ -132,7 +136,7 @@ static const struct {
 };
 
 static const struct {
-    char *opt_name;
+    const char *opt_name;
     int  *opt_addr;
 } options [] = {
     { "show_backups", &show_backups },
@@ -217,7 +221,7 @@ static const struct {
 };
 
 void
-panel_save_setup (struct WPanel *panel, char *section)
+panel_save_setup (struct WPanel *panel, const char *section)
 {
     char buffer [BUF_TINY];
     int  i;
@@ -287,7 +291,7 @@ save_configure (void)
 }
 
 static void
-panel_save_type (char *section, int type)
+panel_save_type (const char *section, int type)
 {
     int i;
 
@@ -299,7 +303,7 @@ panel_save_type (char *section, int type)
 	}
 }
 
-void
+static void
 save_panel_types (void)
 {
     int type;
@@ -327,8 +331,8 @@ save_setup (void)
     save_layout ();
     save_string ("Dirs", "other_dir",
 			       get_other_type () == view_listing
-			       ? opanel->cwd : ".", profile);
-    if (get_current_panel () != NULL)
+			       ? other_panel->cwd : ".", profile);
+    if (current_panel != NULL)
 	    WritePrivateProfileString ("Dirs", "current_is_left",
 				       get_current_index () == 0 ? "1" : "0", profile);
     save_hotlist ();
@@ -355,7 +359,7 @@ save_setup (void)
 }
 
 void
-panel_load_setup (WPanel *panel, char *section)
+panel_load_setup (WPanel *panel, const char *section)
 {
     int i;
     char buffer [BUF_TINY];
@@ -382,16 +386,12 @@ panel_load_setup (WPanel *panel, char *section)
 	}
 
     /* User formats */
-    if (panel->user_format){
-	g_free (panel->user_format);
-	panel->user_format = 0;
-    }
+    g_free (panel->user_format);
     panel->user_format = g_strdup (get_profile_string (section, "user_format",
 						     DEFAULT_USER_FORMAT,
 						     profile_name));
     for (i = 0; i < LIST_TYPES; i++){
-        if (panel->user_status_format [i])
-	    g_free (panel->user_status_format [i]);
+	g_free (panel->user_status_format [i]);
 	g_snprintf (buffer, sizeof (buffer), "user_status%d", i);
 	panel->user_status_format [i] =
 	    g_strdup (get_profile_string (section, buffer,
@@ -415,7 +415,7 @@ load_layout (char *profile_name)
 }
 
 static int
-load_mode (char *section)
+load_mode (const char *section)
 {
     char buffer [20];
     int  i;
@@ -436,7 +436,7 @@ load_mode (char *section)
 
 #ifdef USE_NETCODE
 static char *
-do_load_string (char *s, char *ss, char *def)
+do_load_string (const char *s, const char *ss, const char *def)
 {
     char *buffer = g_malloc (BUF_SMALL);
     char *p;
@@ -523,7 +523,7 @@ load_setup (void)
     load_string ("Misc", "find_ignore_dirs", "", setup_color_string,
 		 sizeof (setup_color_string));
     if (setup_color_string [0])
-	find_ignore_dirs = g_strconcat (":", setup_color_string, ":", NULL);
+	find_ignore_dirs = g_strconcat (":", setup_color_string, ":", (char *) NULL);
 
     /* The default color and the terminal dependent color */
     load_string ("Colors", "base_color", "", setup_color_string,
@@ -579,7 +579,7 @@ void done_setup (void)
 }
 
 static void
-load_keys_from_section (char *terminal, char *profile_name)
+load_keys_from_section (const char *terminal, const char *profile_name)
 {
     char *section_name;
     void *profile_keys;
@@ -589,11 +589,18 @@ load_keys_from_section (char *terminal, char *profile_name)
     if (!terminal)
 	return;
 
-    section_name = g_strconcat ("terminal:", terminal, NULL);
+    section_name = g_strconcat ("terminal:", terminal, (char *) NULL);
     profile_keys = profile_init_iterator (section_name, profile_name);
     g_free (section_name);
     while (profile_keys){
 	profile_keys = profile_iterator_next (profile_keys, &key, &value);
+
+	/* copy=other causes all keys from [terminal:other] to be loaded.  */
+	if (g_strcasecmp (key, "copy") == 0) {
+	    load_keys_from_section (value, profile_name);
+	    continue;
+	}
+
 	key_code = lookup_key (key);
 	if (key_code){
 	    valcopy = convert_controls (value);
