@@ -17,7 +17,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
    This module has been converted to be a widget.
 
@@ -30,38 +30,48 @@
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
-#include "tty.h"
+
 #include "global.h"
+#include "tty.h"
 #include "color.h"
-#include "dialog.h"
+#include "wtools.h"	/* message() */
 #include "dir.h"
-#include "dlg.h"
+#include "dialog.h"
 #include "widget.h"
 #include "panel.h"
 #include "mouse.h"
 #include "main.h"
 #include "file.h"	/* For copy_dir_dir(), move_dir_dir(), erase_dir() */
-#include "fileopctx.h"
 #include "help.h"
 #include "key.h"	/* For mi_getch() */
 #include "tree.h"
+#include "treestore.h"
 #include "cmd.h"
-#include "../vfs/vfs.h"
 
 extern int command_prompt;
 
-#define TREE_NORMALC HOT_FOCUSC
+/* Use the color of the parent widget for the unselected entries */
+#define TREE_NORMALC NORMALC
 
 /* Specifies the display mode: 1d or 2d */
 static int tree_navigation_flag;
 
+struct WTree {
+    Widget widget;
+    struct TreeStore *store;
+    tree_entry *selected_ptr;	/* The selected directory */
+    char search_buffer[256];	/* Current search string */
+    tree_entry **tree_shown;	/* Entries currently on screen */
+    int is_panel;		/* panel or plain widget flag */
+    int active;			/* if it's currently selected */
+    int searching;		/* Are we on searching mode? */
+    int topdiff;		/* The difference between the topmost
+				   shown and the selected */
+};
+
 /* Forwards */
 static void save_tree (WTree *tree);
 static void tree_rescan_cmd (WTree *tree);
-static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par);
-#define tcallback (callback_fn) tree_callback
-
-/* "$Id: tree.c,v 1.29 2001/08/14 00:55:38 proskin Exp $" */
 
 static tree_entry *back_ptr (tree_entry *ptr, int *count)
 {
@@ -87,63 +97,6 @@ static tree_entry *forw_ptr (tree_entry *ptr, int *count)
     return ptr;
 }
 
-#if 0
-/* Add a directory to the list of directories */
-static tree_entry *tree_add_entry (WTree *tree, char *name)
-{
-    if (!tree)
-	return 0;
-
-    return tree_store_add_entry (name);
-}
-
-/* Append a directory to the list of directories */
-static tree_entry *tree_append_entry (WTree *tree, char *name)
-{
-    tree_entry *current, *new;
-    int i, len;
-    int submask = 0;
-
-    /* We assume the directory is not yet in the list */
-
-    new = g_new (tree_entry, 1);
-    if (!tree->store->tree_first){
-        /* Empty list */
-        tree->store->tree_first = new;
-        new->prev = NULL;
-    } else {
-        tree->tree_last->next = new;
-        new->prev = tree->tree_last;
-    }
-    new->next = NULL;
-    tree->store->tree_last = new;
-
-    /* Calculate attributes */
-    new->name = g_strdup (name);
-    len = strlen (new->name);
-    new->sublevel = 0;
-    for (i = 0; i < len; i++)
-	if (new->name [i] == PATH_SEP){
-	    new->sublevel++;
-	    new->subname = new->name + i + 1;
-	}
-    submask = 1 << new->sublevel;
-    submask &= (2 << new->sublevel) - 1;
-    new->submask = submask;
-    new->mark = 0;
-
-    /* Correct the submasks of the previous entries */
-    current = new->prev;
-    while (current && current->sublevel > new->sublevel){
-	current->submask |= 1 << new->sublevel;
-	current = current->prev;
-    }
-
-    /* The entry has now been appended */
-    return new;
-}
-#endif
-
 static void
 remove_callback (tree_entry *entry, void *data)
 {
@@ -167,10 +120,8 @@ static void tree_destroy (WTree *tree)
     tree_store_remove_entry_remove_hook (remove_callback);
     save_tree (tree);
 
-    if (tree->tree_shown){
-	g_free (tree->tree_shown);
-	tree->tree_shown = 0;
-    }
+    g_free (tree->tree_shown);
+    tree->tree_shown = 0;
     tree->selected_ptr = NULL;
 }
 
@@ -220,12 +171,12 @@ static void tree_show_mini_info (WTree *tree, int tree_lines, int tree_cols)
 	attrset (FOCUSC);
 	addch (PATH_SEP);
 
-	addstr (name_trunc (tree->search_buffer, tree_cols-2));
+	addstr ((char *) name_trunc (tree->search_buffer, tree_cols-2));
 	addch (' ');
 	attrset (FOCUSC);
     } else {
 	/* Show full name of selected directory */
-	addstr (name_trunc (tree->selected_ptr->name, tree_cols));
+	addstr ((char *) name_trunc (tree->selected_ptr->name, tree_cols));
     }
 }
 
@@ -249,8 +200,7 @@ static void show_tree (WTree *tree)
 	x = y = 1;
     }
 
-    if (tree->tree_shown)
-	g_free (tree->tree_shown);
+    g_free (tree->tree_shown);
     tree->tree_shown = g_new (tree_entry*, tree_lines);
 
     for (i = 0; i < tree_lines; i++)
@@ -312,7 +262,7 @@ static void show_tree (WTree *tree)
 	    }
 
 	    /* Show full name */
-	    addstr (name_trunc (current->name, tree_cols - 6));
+	    addstr ((char *) name_trunc (current->name, tree_cols - 6));
 	} else{
 	    /* Sub level directory */
 
@@ -346,7 +296,7 @@ static void show_tree (WTree *tree)
 
 	    /* Show sub-name */
 	    addch (' ');
-	    addstr (name_trunc (current->subname,
+	    addstr ((char *) name_trunc (current->subname,
 				tree_cols - 2 - 4 - 3 * j));
 	}
 	addch (' ');
@@ -490,7 +440,7 @@ static void tree_move_to_bottom (WTree *tree)
     tree->topdiff = tlines (tree) - 3 - 1;
 }
 
-void tree_chdir (WTree *tree, char *dir)
+void tree_chdir (WTree *tree, const char *dir)
 {
     tree_entry *current;
 
@@ -502,7 +452,7 @@ void tree_chdir (WTree *tree, char *dir)
 }
 
 void
-sync_tree (char *path)
+sync_tree (const char *path)
 {
     tree_chdir (the_tree, path);
 }
@@ -531,7 +481,7 @@ static void maybe_chdir (WTree *tree)
 static int event_callback (Gpm_Event *event, WTree *tree)
 {
     if (!(event->type & GPM_UP))
-	return MOU_ENDLOOP;
+	return MOU_NORMAL;
 
     if (tree->is_panel)
 	event->y--;
@@ -554,7 +504,7 @@ static int event_callback (Gpm_Event *event, WTree *tree)
 	    chdir_sel (tree);
 	}
     }
-    return MOU_ENDLOOP;
+    return MOU_NORMAL;
 }
 
 /* Search tree for text */
@@ -587,10 +537,10 @@ static int search_tree (WTree *tree, char *text)
 
 static void tree_do_search (WTree *tree, int key)
 {
-    int l;
+    size_t l;
 
     l = strlen (tree->search_buffer);
-    if (l && (key == 8 || key == 0177 || key == KEY_BACKSPACE))
+    if (l && (key == KEY_BACKSPACE))
 	tree->search_buffer [--l] = 0;
     else {
 	if (key && l < sizeof (tree->search_buffer)){
@@ -626,10 +576,10 @@ static int tree_forget_cmd (WTree *tree)
     return 1;
 }
 
-static void tree_copy (WTree *tree, char *default_dest)
+static void tree_copy (WTree *tree, const char *default_dest)
 {
     char   *dest;
-    long   count = 0;
+    off_t  count = 0;
     double bytes = 0;
     FileOpContext *ctx;
 
@@ -647,8 +597,8 @@ static void tree_copy (WTree *tree, char *default_dest)
 	return;
     }
 
-    ctx = file_op_context_new ();
-    file_op_context_create_ui (ctx, OP_COPY, FALSE);
+    ctx = file_op_context_new (OP_COPY);
+    file_op_context_create_ui (ctx, FALSE);
     copy_dir_dir (ctx, tree->selected_ptr->name, dest, 1, 0, 0, 0, &count, &bytes);
     file_op_context_destroy (ctx);
 
@@ -666,12 +616,12 @@ static int tree_copy_cmd (WTree *tree)
     return 1;
 }
 
-static void tree_move (WTree *tree, char *default_dest)
+static void tree_move (WTree *tree, const char *default_dest)
 {
     char   *dest;
     struct stat buf;
     double bytes = 0;
-    long   count = 0;
+    off_t  count = 0;
     FileOpContext *ctx;
 
     if (!tree->selected_ptr)
@@ -692,13 +642,14 @@ static void tree_move (WTree *tree, char *default_dest)
 	return;
     }
     if (!S_ISDIR (buf.st_mode)){
-	message (1, MSG_ERROR, _(" The destination isn't a directory "));
+	file_error (_(" Destination \"%s\" must be a directory \n %s "),
+		    dest);
 	g_free (dest);
 	return;
     }
 
-    ctx = file_op_context_new ();
-    file_op_context_create_ui (ctx, OP_MOVE, FALSE);
+    ctx = file_op_context_new (OP_MOVE);
+    file_op_context_create_ui (ctx, FALSE);
     move_dir_dir (ctx, tree->selected_ptr->name, dest, &count, &bytes);
     file_op_context_destroy (ctx);
 
@@ -736,54 +687,50 @@ tree_mkdir_cmd (WTree *tree)
 static void
 tree_rmdir_cmd (WTree *tree)
 {
-    char old_dir [MC_MAXPATHLEN];
-    long count = 0;
+    off_t count = 0;
     double bytes = 0;
     FileOpContext *ctx;
 
-    if (tree->selected_ptr){
-	if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
-	    return;
-	if (mc_chdir (PATH_SEP_STR))
-	    return;
-	if (confirm_delete){
-	    char *buf;
-	    int result;
-
-	    buf = g_strdup_printf (_("  Delete %s?  "), tree->selected_ptr->name);
-	    result = query_dialog (_(" Delete "), buf, 3, 2, _("&Yes"), _("&No"));
-	    g_free (buf);
-	    if (result != 0){
-		return;
-	    }
-	}
-
-	ctx = file_op_context_new ();
-	file_op_context_create_ui (ctx, OP_DELETE, FALSE);
-	if (erase_dir (ctx, tree->selected_ptr->name, &count, &bytes) == FILE_CONT)
-	    tree_forget_cmd (tree);
-	file_op_context_destroy (ctx);
-	mc_chdir (old_dir);
-	return;
-    } else
+    if (!tree->selected_ptr)
 	return;
+
+    if (confirm_delete) {
+	char *buf;
+	int result;
+
+	buf =
+	    g_strdup_printf (_("  Delete %s?  "),
+			     tree->selected_ptr->name);
+	result =
+	    query_dialog (_(" Delete "), buf, 3, 2, _("&Yes"), _("&No"));
+	g_free (buf);
+	if (result != 0)
+	    return;
+    }
+
+    ctx = file_op_context_new (OP_DELETE);
+    file_op_context_create_ui (ctx, FALSE);
+    if (erase_dir (ctx, tree->selected_ptr->name, &count, &bytes) ==
+	FILE_CONT)
+	tree_forget_cmd (tree);
+    file_op_context_destroy (ctx);
 }
 
-static void set_navig_label (Dlg_head *h);
+static void set_navig_label (WTree *tree);
 
 static void
-tree_toggle_navig (Dlg_head *h)
+tree_toggle_navig (WTree *tree)
 {
     tree_navigation_flag = 1 - tree_navigation_flag;
-    set_navig_label (h);
+    set_navig_label (tree);
 }
 
 static void
-set_navig_label (Dlg_head *h)
+set_navig_label (WTree *tree)
 {
-    define_label_data (h, (Widget *)tree,
-		       4, tree_navigation_flag ? _("Static") : _("Dynamc"),
-		       (void (*)(void *))tree_toggle_navig, h);
+    define_label_data (tree->widget.parent, 4,
+		       tree_navigation_flag ? _("Static") : _("Dynamc"),
+		       (buttonbarfn) tree_toggle_navig, tree);
 }
 
 static void
@@ -863,14 +810,12 @@ move_nextp (WTree *tree)
 static void
 chdir_sel (WTree *tree)
 {
-    if (!tree->is_panel){
-	tree->done = 1;
+    if (!tree->is_panel) {
 	return;
     }
     change_panel ();
-    if (do_cd (tree->selected_ptr->name, cd_exact)){
-	paint_panel (cpanel);
-	select_item (cpanel);
+    if (do_cd (tree->selected_ptr->name, cd_exact)) {
+	select_item (current_panel);
     } else {
 	message (1, MSG_ERROR, _(" Cannot chdir to \"%s\" \n %s "),
 		 tree->selected_ptr->name, unix_error_string (errno));
@@ -908,7 +853,13 @@ tree_start_search (WTree *tree)
     }
 }
 
-static const key_map tree_keymap [] = {
+typedef void (*tree_key_action) (WTree *);
+typedef struct {
+    int key_code;
+    tree_key_action fn;
+} tree_key_map;
+
+static const tree_key_map tree_keymap [] = {
     { XCTRL('n'), move_down    },
     { XCTRL('p'), move_up      },
     { KEY_DOWN,   move_down    },
@@ -916,9 +867,11 @@ static const key_map tree_keymap [] = {
     { '\n',       chdir_sel    },
     { KEY_ENTER,  chdir_sel    },
     { KEY_HOME,   move_home    },
-    { KEY_C1,     move_end     },
-    { KEY_END,    move_end     },
     { KEY_A1,     move_home    },
+    { ALT ('<'),  move_home    },
+    { KEY_END,    move_end     },
+    { KEY_C1,     move_end     },
+    { ALT ('>'),  move_end     },
     { KEY_NPAGE,  move_nextp   },
     { KEY_PPAGE,  move_prevp   },
     { XCTRL('v'), move_nextp   },
@@ -932,7 +885,7 @@ static const key_map tree_keymap [] = {
     { 0, 0 }
     };
 
-static inline int
+static inline cb_ret_t
 tree_key (WTree *tree, int key)
 {
     int i;
@@ -943,7 +896,7 @@ tree_key (WTree *tree, int key)
 	        tree->searching = 0;
 	    (*tree_keymap [i].fn)(tree);
 	    show_tree (tree);
-	    return 1;
+	    return MSG_HANDLED;
 	}
     }
 
@@ -959,29 +912,30 @@ tree_key (WTree *tree, int key)
 	if (tree->is_panel) {
 	    tree->searching = 0;
 	    show_tree (tree);
-	    return 1;  /* eat abort char */
+	    return MSG_HANDLED;  /* eat abort char */
 	}
-	return 0;  /* modal tree dialog: let upper layer see the
-		      abort character and close the dialog */
+	/* modal tree dialog: let upper layer see the
+	   abort character and close the dialog */
+	return MSG_NOT_HANDLED;
     }
 
     /* Do not eat characters not meant for the tree below ' ' (e.g. C-l). */
-    if ((key >= ' '&& key <= 255) || key == 8 || key == KEY_BACKSPACE) {
+    if ((key >= ' ' && key <= 255) || key == KEY_BACKSPACE) {
 	if (tree->searching){
 	    tree_do_search (tree, key);
 	    show_tree (tree);
-	    return 1;
+	    return MSG_HANDLED;
 	}
 
 	if (!command_prompt) {
 	    tree_start_search (tree);
 	    tree_do_search (tree, key);
-	    return 1;
+	    return MSG_HANDLED;
 	}
 	return tree->is_panel;
     }
 
-    return 0;
+    return MSG_NOT_HANDLED;
 }
 
 static void
@@ -1000,55 +954,64 @@ tree_frame (Dlg_head *h, WTree *tree)
 }
 
 
-static int
-tree_callback (Dlg_head *h, WTree *tree, int msg, int par)
+static cb_ret_t
+tree_callback (WTree *tree, widget_msg_t msg, int parm)
 {
-    switch (msg){
+    Dlg_head *h = tree->widget.parent;
+
+    switch (msg) {
     case WIDGET_DRAW:
 	tree_frame (h, tree);
 	show_tree (tree);
-	return 1;
+	return MSG_HANDLED;
 
     case WIDGET_KEY:
-	return tree_key (tree, par);
+	return tree_key (tree, parm);
 
     case WIDGET_FOCUS:
 	tree->active = 1;
-	define_label (h, (Widget *)tree, 1, _("Help"), (voidfn) tree_help_cmd);
-	define_label_data (h, (Widget *)tree,
-	    2, _("Rescan"), (buttonbarfn)tree_rescan_cmd, tree);
-	define_label_data (h, (Widget *)tree,
-	    3, _("Forget"), (buttonbarfn)tree_forget_cmd, tree);
-	define_label_data (h, (Widget *)tree,
-	    5, _("Copy"),   (buttonbarfn) tree_copy_cmd, tree);
-	define_label_data (h, (Widget *)tree,
-	    6, _("RenMov"), (buttonbarfn) tree_move_cmd, tree);
+	define_label (h, 1, _("Help"), (voidfn) tree_help_cmd);
+	define_label_data (h, 2, _("Rescan"),
+			   (buttonbarfn) tree_rescan_cmd, tree);
+	define_label_data (h, 3, _("Forget"),
+			   (buttonbarfn) tree_forget_cmd, tree);
+	define_label_data (h, 5, _("Copy"), (buttonbarfn) tree_copy_cmd,
+			   tree);
+	define_label_data (h, 6, _("RenMov"), (buttonbarfn) tree_move_cmd,
+			   tree);
 #if 0
 	/* FIXME: mkdir is currently defunct */
-	define_label_data (h, (Widget *)tree,
-	    7, _("Mkdir"),  (buttonbarfn) tree_mkdir_cmd, tree);
+	define_label_data (h, 7, _("Mkdir"), (buttonbarfn) tree_mkdir_cmd,
+			   tree);
 #else
-        define_label (h, (Widget *)tree, 7, "", 0);
+	define_label (h, 7, "", 0);
 #endif
-	define_label_data (h, (Widget *)tree,
-	    8, _("Rmdir"),  (buttonbarfn) tree_rmdir_cmd, tree);
-	set_navig_label (h);
-	redraw_labels (h, (Widget *)tree);
+	define_label_data (h, 8, _("Rmdir"), (buttonbarfn) tree_rmdir_cmd,
+			   tree);
+	set_navig_label (tree);
+	redraw_labels (h);
 
 
 	/* FIXME: Should find a better way of only displaying the
 	   currently selected item */
 	show_tree (tree);
-	return 1;
+	return MSG_HANDLED;
 
 	/* FIXME: Should find a better way of changing the color of the
 	   selected item */
+
     case WIDGET_UNFOCUS:
 	tree->active = 0;
 	show_tree (tree);
-	return 1;
+	return MSG_HANDLED;
+
+    case WIDGET_DESTROY:
+	tree_destroy (tree);
+	return MSG_HANDLED;
+
+    default:
+	return default_proc (msg, parm);
     }
-    return default_proc (h, msg, par);
 }
 
 WTree *
@@ -1056,18 +1019,17 @@ tree_new (int is_panel, int y, int x, int lines, int cols)
 {
     WTree *tree = g_new (WTree, 1);
 
-    init_widget (&tree->widget, y, x, lines, cols, tcallback,
-		 (destroy_fn) tree_destroy, (mouse_h) event_callback, NULL);
+    init_widget (&tree->widget, y, x, lines, cols,
+		 (callback_fn) tree_callback, (mouse_h) event_callback);
     tree->is_panel = is_panel;
     tree->selected_ptr = 0;
 
     tree->store = tree_store_get ();
     tree_store_add_entry_remove_hook (remove_callback, tree);
     tree->tree_shown = 0;
-    tree->search_buffer [0] = 0;
+    tree->search_buffer[0] = 0;
     tree->topdiff = tree->widget.lines / 2;
     tree->searching = 0;
-    tree->done = 0;
     tree->active = 0;
 
     /* We do not want to keep the cursor */
@@ -1075,3 +1037,11 @@ tree_new (int is_panel, int y, int x, int lines, int cols)
     load_tree (tree);
     return tree;
 }
+
+/* Return name of the currently selected entry */
+char *
+tree_selected_name (WTree *tree)
+{
+    return tree->selected_ptr->name;
+}
+
