@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
 #include <stdio.h>
@@ -23,7 +23,6 @@
 
 #include "global.h"
 #include "tty.h"
-#include "dialog.h"
 #include "color.h"
 #include "dir.h"
 #include "panel.h"
@@ -31,14 +30,14 @@
 #include "subshell.h" 		/* for subshell_pty */
 #include "user.h"
 #include "layout.h"
+#include "execute.h"
 #include "setup.h"
 
-#include "edit/edit.h"		/* BLOCK_FILE */
-#include "edit/edit-widget.h"	/* WEdit */
-#include "../vfs/vfs.h"
+#include "../edit/edit.h"		/* BLOCK_FILE */
+#include "../edit/edit-widget.h"	/* WEdit */
 
 /* For the simple listbox manager */
-#include "dlg.h"
+#include "dialog.h"
 #include "widget.h"
 #include "wtools.h"
 
@@ -117,7 +116,7 @@ int check_format_var (const char *p, char **v)
 {
     const char *q = p;
     char *var_name;
-    char *value;
+    const char *value;
     const char *dots = 0;
     
     *v = 0;
@@ -138,19 +137,15 @@ int check_format_var (const char *p, char **v)
 	}
 	
 	/* Copy the variable name */
-	var_name = g_malloc (dots - p);
-	strncpy (var_name, p+4, dots-2 - (p+3));
-	var_name [dots-2 - (p+3)] = 0;
+	var_name = g_strndup (p + 4, dots-2 - (p+3));
 
 	value = getenv (var_name);
 	g_free (var_name);
 	if (value){
 	    *v = g_strdup (value);
 	    return q-p;
-	}
-	var_name = g_malloc (q - dots + 1);
-	strncpy (var_name, dots, q - dots + 1);
-	var_name [q-dots] = 0;
+	}	
+	var_name = g_strndup (dots, q - dots);
 	*v = var_name;
 	return q-p;
     }
@@ -175,39 +170,59 @@ strip_ext(char *ss)
 char *
 expand_format (struct WEdit *edit_widget, char c, int quote)
 {
-    WPanel *panel;
+    WPanel *panel = NULL;
     char *(*quote_func) (const char *, int);
     char *fname;
+    char *result;
+    char c_lc;
 
     if (c == '%')
 	return g_strdup ("%");
 
-    if (islower ((unsigned) c))
-	panel = cpanel;
+    if (edit_one_file != NULL)
+	fname = edit_widget->filename;
     else {
-	if (get_other_type () != view_listing)
-	    return g_strdup ("");
-	panel = other_panel;
+	if (islower ((unsigned char) c))
+	    panel = current_panel;
+	else {
+	    if (get_other_type () != view_listing)
+		return g_strdup ("");
+	    panel = other_panel;
+	}
+	fname = panel->dir.list[panel->selected].fname;
     }
-    if (!panel)
-	panel = cpanel;
 
     if (quote)
 	quote_func = name_quote;
     else
 	quote_func = fake_name_quote;
 
-    c = tolower (c);
-    fname = panel->dir.list[panel->selected].fname;
+    c_lc = tolower ((unsigned char) c);
 
-    switch (c) {
+    switch (c_lc) {
     case 'f':
     case 'p':
 	return (*quote_func) (fname, 0);
     case 'x':
 	return (*quote_func) (extension (fname), 0);
     case 'd':
-	return (*quote_func) (panel->cwd, 0);
+	{
+	    char *cwd;
+	    char *qstr;
+
+	    cwd = g_malloc(MC_MAXPATHLEN + 1);
+
+	    if (panel)
+		g_strlcpy(cwd, panel->cwd, MC_MAXPATHLEN + 1);
+	    else
+		mc_get_current_wd(cwd, MC_MAXPATHLEN + 1);
+
+	    qstr = (*quote_func) (cwd, 0);
+
+	    g_free (cwd);
+
+	    return qstr;
+	}
     case 'i':			/* indent equal number cursor position in line */
 	if (edit_widget)
 	    return g_strnfill (edit_widget->curs_col, ' ');
@@ -219,11 +234,11 @@ expand_format (struct WEdit *edit_widget, char c, int quote)
     case 'k':			/* block file name */
     case 'b':			/* block file name / strip extension */  {
 	    if (edit_widget) {
-		char *file = g_strconcat (home_dir, BLOCK_FILE, NULL);
+		char *file = g_strconcat (home_dir, BLOCK_FILE, (char *) NULL);
 		fname = (*quote_func) (file, 0);
 		g_free (file);
 		return fname;
-	    } else if (c == 'b') {
+	    } else if (c_lc == 'b') {
 		return strip_ext ((*quote_func) (fname, 0));
 	    }
 	    break;
@@ -237,7 +252,7 @@ expand_format (struct WEdit *edit_widget, char c, int quote)
 	    return (*quote_func) (menu, 0);
 	break;
     case 's':
-	if (!panel->marked)
+	if (!panel || !panel->marked)
 	    return (*quote_func) (fname, 0);
 
 	/* Fall through */
@@ -248,6 +263,9 @@ expand_format (struct WEdit *edit_widget, char c, int quote)
 	    int length = 2, i;
 	    char *block, *tmp;
 
+	    if (!panel)
+		return g_strdup ("");
+
 	    for (i = 0; i < panel->count; i++)
 		if (panel->dir.list[i].f.marked)
 		    length += strlen (panel->dir.list[i].fname) + 1;	/* for space */
@@ -260,13 +278,15 @@ expand_format (struct WEdit *edit_widget, char c, int quote)
 			    (*quote_func) (panel->dir.list[i].fname, 0));
 		    g_free (tmp);
 		    strcat (block, " ");
-		    if (c == 'u')
+		    if (c_lc == 'u')
 			do_file_mark (panel, i, 0);
 		}
 	    return block;
 	}			/* sub case block */
     }				/* switch */
-    return g_strdup ("");
+    result = g_strdup ("% ");
+    result[1] = c;
+    return result;
 }
 
 /*
@@ -300,13 +320,15 @@ check_patterns (char *p)
 
 /* Copies a whitespace separated argument from p to arg. Returns the
    point after argument. */
-static char *extract_arg (char *p, char *arg)
+static char *extract_arg (char *p, char *arg, int size)
 {
     while (*p && (*p == ' ' || *p == '\t' || *p == '\n'))
 	p++;
                 /* support quote space .mnu */
-    while (*p && (*p != ' ' || *(p-1) == '\\') && *p != '\t' && *p != '\n')
+    while (size > 1 && *p && (*p != ' ' || *(p-1) == '\\') && *p != '\t' && *p != '\n') {
 	*arg++ = *p++;
+	size--;
+    }
     *arg = 0;
     if (!*p || *p == '\n')
 	p --;
@@ -318,7 +340,7 @@ static char *extract_arg (char *p, char *arg)
 static int test_type (WPanel *panel, char *arg)
 {
     int result = 0; /* False by default */
-    int st_mode = panel->dir.list [panel->selected].buf.st_mode;
+    int st_mode = panel->dir.list [panel->selected].st.st_mode;
 
     for (;*arg != 0; arg++){
 	switch (*arg){
@@ -373,7 +395,7 @@ static char *test_condition (WEdit *edit_widget, char *p, int *condition)
 	if ((*p == ' ' && *(p-1) != '\\') || *p == '\t')
 	    continue;
 	if (*p >= 'a')
-	    panel = cpanel;
+	    panel = current_panel;
 	else {
 	    if (get_other_type () == view_listing)
 		panel = other_panel;
@@ -389,29 +411,29 @@ static char *test_condition (WEdit *edit_widget, char *p, int *condition)
 	    p--;
 	    break;
 	case 'f': /* file name pattern */
-	    p = extract_arg (p, arg);
+	    p = extract_arg (p, arg, sizeof (arg));
 	    *condition = panel && regexp_match (arg, panel->dir.list [panel->selected].fname, match_file);
 	    break;
 	case 'y': /* syntax pattern */
             if (edit_widget && edit_widget->syntax_type) {
-	        p = extract_arg (p, arg);
+	        p = extract_arg (p, arg, sizeof (arg));
 	        *condition = panel &&
                     regexp_match (arg, edit_widget->syntax_type, match_normal);
 	    }
-                break;
+            break;
 	case 'd':
-	    p = extract_arg (p, arg);
+	    p = extract_arg (p, arg, sizeof (arg));
 	    *condition = panel && regexp_match (arg, panel->cwd, match_file);
 	    break;
 	case 't':
-	    p = extract_arg (p, arg);
+	    p = extract_arg (p, arg, sizeof (arg));
 	    *condition = panel && test_type (panel, arg);
 	    break;
 	case 'x': /* executable */
 	{
 	    struct stat status;
 	    
-	    p = extract_arg (p, arg);
+	    p = extract_arg (p, arg, sizeof (arg));
 	    if (stat (arg, &status) == 0)
 		*condition = is_exe (status.st_mode);
 	    else
@@ -431,50 +453,45 @@ static char *test_condition (WEdit *edit_widget, char *p, int *condition)
 static void
 debug_out (char *start, char *end, int cond)
 {
-    static char msg [256];
+    static char *msg;
     int len;
 
     if (start == NULL && end == NULL){
-	if (cond == 0){
-	    /* Init */
-	    msg [0] = 0;
-	} else {
-	    /* Show output */
-	    if (!debug_flag)
-		return;
+	/* Show output */
+	if (debug_flag && msg) {
 	    len = strlen (msg);
 	    if (len)
 		msg [len - 1] = 0;
 	    message (0, _(" Debug "), "%s", msg);
-	    debug_flag = 0;
+
 	}
+	debug_flag = 0;
+	g_free (msg);
+	msg = NULL;
     } else {
+	const char *type;
+	char *p;
+
 	/* Save debug info for later output */
 	if (!debug_flag)
 	    return;
 	/* Save the result of the condition */
 	if (debug_error){
-	    strcat (msg, _(" ERROR: "));
+	    type = _(" ERROR: ");
 	    debug_error = 0;
 	}
 	else if (cond)
-	    strcat (msg, _(" True:  "));
+	    type = _(" True:  ");
 	else
-	    strcat (msg, _(" False: "));
-	/* Copy condition statement */
-	len = strlen (msg);
-	if (end == NULL){
-	    /* Copy one character */
-	    msg [len] = *start;
-	    msg [len + 1] = 0;
-	} else {
-	    /* Copy many characters */
-	    while (start < end){
-		msg [len++] = *start++;
-	    }
-	    msg [len] = 0;
-	}
-	strcat (msg, " \n");
+	    type = _(" False: ");
+	/* This is for debugging, don't need to be super efficient.  */
+	if (end == NULL)
+	    p = g_strdup_printf ("%s%s%c \n", msg ? msg : "", type, *start);
+	else
+	    p = g_strdup_printf ("%s%s%.*s \n", msg ? msg : "", type,
+			(int) (end - start), start);
+	g_free (msg);
+	msg = p;
     }
 }
 
@@ -486,8 +503,6 @@ static char *test_line (WEdit *edit_widget, char *p, int *result)
     char operator;
     char *debug_start, *debug_end;
 
-    /* Init debugger */
-    debug_out (NULL, NULL, 0);
     /* Repeat till end of line */
     while (*p && *p != '\n') {
         /* support quote space .mnu */
@@ -543,7 +558,7 @@ static char *test_line (WEdit *edit_widget, char *p, int *result)
 
 /* FIXME: recode this routine on version 3.0, it could be cleaner */
 static void
-execute_menu_command (WEdit *edit_widget, char *commands)
+execute_menu_command (WEdit *edit_widget, const char *commands)
 {
     FILE *cmd_file;
     int  cmd_file_fd;
@@ -553,16 +568,15 @@ execute_menu_command (WEdit *edit_widget, char *commands)
     char prompt [80];
     int  col;
     char *file_name;
-#ifdef NATIVE_WIN32
-    char *p;
-#endif
+    int run_view = 0;
+
     /* Skip menu entry title line */
     commands = strchr (commands, '\n');
     if (!commands){
 	return;
     }
 
-    cmd_file_fd = mc_mkstemps(&file_name, "mcusr", SCRIPT_SUFFIX);
+    cmd_file_fd = mc_mkstemps (&file_name, "mcusr", SCRIPT_SUFFIX);
 
     if (cmd_file_fd == -1){
 	message (1, MSG_ERROR, _(" Cannot create temporary command file \n %s "),
@@ -570,6 +584,7 @@ execute_menu_command (WEdit *edit_widget, char *commands)
 	return;
     }
     cmd_file = fdopen (cmd_file_fd, "w");
+    fputs ("#! /bin/sh\n", cmd_file);
     commands++;
     
     for (col = 0; *commands; commands++){
@@ -578,6 +593,8 @@ execute_menu_command (WEdit *edit_widget, char *commands)
 		break;
 	    while (*commands == ' ' || *commands == '\t')
 	        commands++;
+	    if (*commands == 0)
+		break;
 	}
 	col++;
 	if (*commands == '\n')
@@ -622,15 +639,26 @@ execute_menu_command (WEdit *edit_widget, char *commands)
 	    }
 	} else {
 	    if (*commands == '%') {
-		do_quote = 1; /* Default: Quote expanded macro */
-		expand_prefix_found = 1;
+		int i = check_format_view (commands + 1);
+		if (i) {
+		    commands += i;
+		    run_view = 1;
+		} else {
+		    do_quote = 1; /* Default: Quote expanded macro */
+		    expand_prefix_found = 1;
+		}
 	    } else
 		fputc (*commands, cmd_file);
 	}
     }
     fclose (cmd_file);
     chmod (file_name, S_IRWXU);
-    execute (file_name);
+    if (run_view) {
+	run_view = 0;
+	view (file_name, 0, &run_view, 0);
+    } else {
+	shell_execute (file_name, EXECUTE_HIDE);
+    }
     unlink (file_name);
     g_free (file_name);
 }
@@ -734,7 +762,7 @@ user_menu_cmd (struct WEdit *edit_widget)
 	    } else if (*p == '+'){
 		if (*(p+1) == '='){
 		    /* Combined adding and default */
-		    p = test_line (edit_widget, p, &accept_entry);
+		    p = test_line (edit_widget, p + 1, &accept_entry);
 		    if (selected == 0 && accept_entry)
 			selected = menu_lines;
 		} else {
@@ -744,7 +772,7 @@ user_menu_cmd (struct WEdit *edit_widget)
 	    } else if (*p == '='){
 		if (*(p+1) == '+'){
 		    /* Combined adding and default */
-		    p = test_line (edit_widget, p, &accept_entry);
+		    p = test_line (edit_widget, p + 1, &accept_entry);
 		    if (selected == 0 && accept_entry)
 			selected = menu_lines;
 		} else {
@@ -806,8 +834,7 @@ user_menu_cmd (struct WEdit *edit_widget)
     easy_patterns = old_patterns;
     g_free (menu);
     menu = NULL;
-    if (entries)
-	g_free (entries);
+    g_free (entries);
     g_free (data);
 }
 
