@@ -13,173 +13,101 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include "tty.h"
 #include <stdio.h>
 #include <string.h>
-#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(HAS_NO_TERMIOS_H)
+#ifdef HAVE_TERMIOS_H
 #    include <termios.h>
 #endif
+
 #include "global.h"
+#include "tty.h"
 #include "color.h"
 #include "mouse.h"
-#include "dlg.h"
+#include "dialog.h"
 #include "widget.h"
 #include "win.h"
 #include "key.h"		/* XCTRL and ALT macros  */
 #include "layout.h"
 
-/* "$Id: win.c,v 1.7 2001/06/06 21:41:44 proskin Exp $" */
-
-#ifndef HAVE_X
-typedef void (*fnptr)(void);
-
-typedef struct Fkey_Table_List {
-    fnptr  actions[11];
-    struct Fkey_Table_List *next;
-    int    has_labels;
-} Fkey_Table_List;
-
-static Fkey_Table_List *fkey_table_list = NULL;
-
-/* Width of output is always seven characters */
-void sprint_bytesize (char *buffer, int size, int scale)
+/*
+ * Common handler for standard movement keys in a text area.  Provided
+ * functions are called with the "data" argument.  backfn and forfn also
+ * get an argument indicating how many lines to scroll.  Return 1 if
+ * the key was handled, 0 otherwise.
+ */
+int
+check_movement_keys (int key, int page_size, const void *data, movefn backfn,
+		     movefn forfn, movefn topfn, movefn bottomfn)
 {
-    char scales[] = " kMGT";
-
-    if (size > 0){
-	while (size > 9999 && scale < sizeof (scales)){
-	    size = (size + 512) / 1024;
-	    scale ++;
-	}
-    }
-    if (scale > 0)
-	g_snprintf (buffer, 10, "%4d %cb", size, scales[scale]);
-    else
-	g_snprintf (buffer, 10, "%4d b ", size);
-}
-
-void print_bytesize (int size, int scale)
-{
-    char buffer [10];
-    
-    sprint_bytesize (buffer, size, scale);
-    printw (buffer);
-}
-
-/* Return values: 0 = not a fkey, other = was a fkey */
-int check_fkeys (int c)
-{
-    int fkey;
-
-    if (!fkey_table_list)
-	return 0;
-    
-    switch (c){
-    case KEY_F(1):
-	fkey = 1;
-	break;
-    case KEY_F(2):
-	fkey = 2;
-	break;
-    case KEY_F(3):
-	fkey = 3;
-	break;
-    case KEY_F(4):
-	fkey = 4;
-	break;
-    case KEY_F(5):
-	fkey = 5;
-	break;
-    case KEY_F(6):
-	fkey = 6;
-	break;
-    case KEY_F(7):
-	fkey = 7;
-	break;
-    case KEY_F(8):
-	fkey = 8;
-	break;
-    case KEY_F(9):
-	fkey = 9;
-	break;
-    case KEY_F(10):
-	fkey = 10;
-	break;
-    default:
-	return 0;
-    }
-    if (fkey_table_list->actions [fkey]){
-	fkey_table_list->actions [fkey] ();
-	return fkey;
-    }
-    else
-	return 0;
-}
-#endif /* !HAVE_X */
-
-/* Return values: 0 = not a movement key, 1 = was a movement key */
-int check_movement_keys (int c, int additional, int page_size, void *data,
-			 movefn backfn, movefn forfn, movefn topfn,
-			 movefn bottomfn)
-{
-    switch (c){
+    switch (key) {
     case KEY_UP:
     case XCTRL ('p'):
-	(*backfn)(data, 1);
-	return 1;
-	
+	(*backfn) (data, 1);
+	break;
+
     case KEY_DOWN:
     case XCTRL ('n'):
-	(*forfn)(data, 1);
-	return 1;
-	
+	(*forfn) (data, 1);
+	break;
+
     case KEY_PPAGE:
-    case ALT('v'):
-	(*backfn)(data, page_size-1);
-	return 1;
-	
+    case ALT ('v'):
+	(*backfn) (data, page_size - 1);
+	break;
+
     case KEY_NPAGE:
-    case XCTRL('v'):
-	(*forfn)(data, page_size-1);
-	return 1;
-	
+    case XCTRL ('v'):
+	(*forfn) (data, page_size - 1);
+	break;
+
     case KEY_HOME:
+    case KEY_M_CTRL | KEY_HOME:
+    case KEY_M_CTRL | KEY_PPAGE:
     case KEY_A1:
-	(*topfn)(data, 0);
-	return 1;
+    case ALT ('<'):
+	(*topfn) (data, 0);
+	break;
+
     case KEY_END:
+    case KEY_M_CTRL | KEY_END:
+    case KEY_M_CTRL | KEY_NPAGE:
     case KEY_C1:
-	(*bottomfn)(data, 0);
-	return 1;
+    case ALT ('>'):
+	(*bottomfn) (data, 0);
+	break;
+
+    case 'b':
+    case KEY_BACKSPACE:
+	(*backfn) (data, page_size - 1);
+	break;
+
+    case ' ':
+	(*forfn) (data, page_size - 1);
+	break;
+
+    case 'u':
+	(*backfn) (data, page_size / 2);
+	break;
+
+    case 'd':
+	(*forfn) (data, page_size / 2);
+	break;
+
+    case 'g':
+	(*topfn) (data, 0);
+	break;
+
+    case 'G':
+	(*bottomfn) (data, 0);
+	break;
+
+    default:
+	return MSG_NOT_HANDLED;
     }
-    if (additional)
-        switch (c){
-	case 'b':  
-	case XCTRL('h'):
-	case KEY_BACKSPACE:
-	case 0177:
-	    (*backfn)(data, page_size-1);
-	    return 1;
-	case ' ':
-	    (*forfn)(data, page_size-1);
-	    return 1;
-	case 'u':
-	    (*backfn)(data, page_size / 2);
-	    return 1;
-	case 'd':
-	    (*forfn)(data, page_size / 2);
-	    return 1;
-	case 'g':
-	    (*topfn)(data, 0);
-	    return 1;
-	case 'G':
-	    (*bottomfn)(data, 0);
-	    return 1;
-	}
-    return 0;
+    return MSG_HANDLED;
 }
 
 /* Classification routines */
@@ -189,7 +117,6 @@ int is_abort_char (int c)
 	    c == KEY_F(10));
 }
 
-#ifndef HAVE_X
 void mc_raw_mode (void)
 {
     raw ();
@@ -200,9 +127,26 @@ void mc_noraw_mode (void)
     noraw ();
 }
 
-int is_quit_char (int c)
+/* This flag is set by xterm detection routine in function main() */
+/* It is used by function view_other_cmd() */
+int xterm_flag = 0;
+
+/* The following routines only work on xterm terminals */
+
+void do_enter_ca_mode (void)
 {
-    return (c == XCTRL('g') || (c == ESC_CHAR) || (c == KEY_F(10)));
+    if (!xterm_flag)
+	return;
+    fprintf (stdout, /* ESC_STR ")0" */ ESC_STR "7" ESC_STR "[?47h");
+    fflush (stdout);
+}
+
+void do_exit_ca_mode (void)
+{
+    if (!xterm_flag)
+	return;
+    fprintf (stdout, ESC_STR "[?47l" ESC_STR "8" ESC_STR "[m");
+    fflush (stdout);
 }
 
 /* This table is a mapping between names and the constants we use
@@ -261,7 +205,7 @@ key_code_name_t key_name_conv_tab [] = {
     { (int) '\n',    "kpenter",    N_("Enter on keypad") },
     { (int) '/',     "kpslash",    N_("Slash on keypad") },
     { (int) '#',     "kpnumlock",  N_("NumLock on keypad") },
-    { 0, 0 }
+    { 0, 0, 0 }
     };
 
 /* Return the code associated with the symbolic name keyname */
@@ -277,4 +221,3 @@ int lookup_key (char *keyname)
     return 0;
 }
 
-#endif /* !HAVE_X */
