@@ -16,19 +16,21 @@
  * saves both computer's memory and my work.  <pavel@ucw.cz>
  * */
     
+static struct vfs_class vfs_local_ops;
+
 static void *
-local_open (vfs *me, char *file, int flags, int mode)
+local_open (struct vfs_class *me, const char *file, int flags, int mode)
 {
     int *local_info;
     int fd;
 
-    fd = open (file, NO_LINEAR(flags), mode);
+    fd = open (file, NO_LINEAR (flags), mode);
     if (fd == -1)
 	return 0;
 
     local_info = g_new (int, 1);
     *local_info = fd;
-    
+
     return local_info;
 }
 
@@ -66,13 +68,13 @@ local_close (void *data)
 }
 
 int
-local_errno (vfs *me)
+local_errno (struct vfs_class *me)
 {
     return errno;
 }
 
 static void *
-local_opendir (vfs *me, char *dirname)
+local_opendir (struct vfs_class *me, const char *dirname)
 {
     DIR **local_info;
     DIR *dir;
@@ -87,29 +89,6 @@ local_opendir (vfs *me, char *dirname)
     return local_info;
 }
 
-static int
-local_telldir (void *data)
-{
-#ifdef HAVE_TELLDIR
-    return telldir( *(DIR **) data );
-#else
- #warning "Native telldir() not available, emulation not implemented"
-    abort();
-    return 0; /* for dumb compilers */
-#endif /* !HAVE_TELLDIR */
-}
-
-static void
-local_seekdir (void *data, int offset)
-{
-#ifdef HAVE_SEEKDIR
-    seekdir( *(DIR **) data, offset );
-#else
- #warning "Native seekdir() not available, emulation not implemented"
-    abort();
-#endif /* !HAVE_SEEKDIR */
-}
-
 static void *
 local_readdir (void *data)
 {
@@ -122,19 +101,18 @@ local_closedir (void *data)
     int i;
 
     i = closedir (* (DIR **) data);
-    if (data)
-	g_free (data);
+    g_free (data);
     return i;
 }
 
 static int
-local_stat (vfs *me, char *path, struct stat *buf)
+local_stat (struct vfs_class *me, const char *path, struct stat *buf)
 {
     return stat (path, buf);
 }
 
 static int
-local_lstat (vfs *me, char *path, struct stat *buf)
+local_lstat (struct vfs_class *me, const char *path, struct stat *buf)
 {
 #ifndef HAVE_STATLSTAT
     return lstat (path,buf);
@@ -150,43 +128,43 @@ local_fstat (void *data, struct stat *buf)
 }
 
 static int
-local_chmod (vfs *me, char *path, int mode)
+local_chmod (struct vfs_class *me, const char *path, int mode)
 {
     return chmod (path, mode);
 }
 
 static int
-local_chown (vfs *me, char *path, int owner, int group)
+local_chown (struct vfs_class *me, const char *path, int owner, int group)
 {
     return chown (path, owner, group);
 }
 
 static int
-local_utime (vfs *me, char *path, struct utimbuf *times)
+local_utime (struct vfs_class *me, const char *path, struct utimbuf *times)
 {
     return utime (path, times);
 }
 
 static int
-local_readlink (vfs *me, char *path, char *buf, int size)
+local_readlink (struct vfs_class *me, const char *path, char *buf, size_t size)
 {
     return readlink (path, buf, size);
 }
 
 static int
-local_unlink (vfs *me, char *path)
+local_unlink (struct vfs_class *me, const char *path)
 {
     return unlink (path);
 }
 
 static int
-local_symlink (vfs *me, char *n1, char *n2)
+local_symlink (struct vfs_class *me, const char *n1, const char *n2)
 {
     return symlink (n1, n2);
 }
 
 static int
-local_write (void *data, char *buf, int nbyte)
+local_write (void *data, const char *buf, int nbyte)
 {
     int fd;
     int n;
@@ -208,13 +186,13 @@ local_write (void *data, char *buf, int nbyte)
 }
 
 static int
-local_rename (vfs *me, char *a, char *b)
+local_rename (struct vfs_class *me, const char *a, const char *b)
 {
     return rename (a, b);
 }
 
 static int
-local_chdir (vfs *me, char *path)
+local_chdir (struct vfs_class *me, const char *path)
 {
     return chdir (path);
 }
@@ -228,62 +206,45 @@ local_lseek (void *data, off_t offset, int whence)
 }
 
 static int
-local_mknod (vfs *me, char *path, int mode, int dev)
+local_mknod (struct vfs_class *me, const char *path, int mode, int dev)
 {
     return mknod (path, mode, dev);
 }
 
 static int
-local_link (vfs *me, char *p1, char *p2)
+local_link (struct vfs_class *me, const char *p1, const char *p2)
 {
     return link (p1, p2);
 }
 
 static int
-local_mkdir (vfs *me, char *path, mode_t mode)
+local_mkdir (struct vfs_class *me, const char *path, mode_t mode)
 {
     return mkdir (path, mode);
 }
 
 static int
-local_rmdir (vfs *me, char *path)
+local_rmdir (struct vfs_class *me, const char *path)
 {
     return rmdir (path);
 }
 
-static vfsid
-local_getid (vfs *me, char *path, struct vfs_stamping **parent)
-{
-    *parent = NULL;
-    return (vfsid) -1; /* We do not free local fs stuff at all */
-}
-
-static int
-local_nothingisopen (vfsid id)
-{
-    return 0;
-}
-
-static void
-local_free (vfsid id)
-{
-}
-
 static char *
-local_getlocalcopy (vfs *me, char *path)
+local_getlocalcopy (struct vfs_class *me, const char *path)
 {
     return g_strdup (path);
 }
 
 static int
-local_ungetlocalcopy (vfs *me, char *path, char *local, int has_changed)
+local_ungetlocalcopy (struct vfs_class *me, const char *path,
+		      const char *local, int has_changed)
 {
     return 0;
 }
 
 #ifdef HAVE_MMAP
 caddr_t
-local_mmap (vfs *me, caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
+local_mmap (struct vfs_class *me, caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
 {
     int fd = * (int *)data;
 
@@ -291,134 +252,53 @@ local_mmap (vfs *me, caddr_t addr, size_t len, int prot, int flags, void *data,
 }
 
 int
-local_munmap (vfs *me, caddr_t addr, size_t len, void *data)
+local_munmap (struct vfs_class *me, caddr_t addr, size_t len, void *data)
 {
     return munmap (addr, len);
 }
 #endif
 
 static int
-local_which (vfs *me, char *path)
+local_which (struct vfs_class *me, const char *path)
 {
     return 0;		/* Every path which other systems do not like is expected to be ours */
 }
 
-vfs vfs_local_ops = {
-    NULL,	/* This is place of next pointer */
-    "localfs",
-    0,		/* flags */
-    NULL,	/* prefix */
-    NULL,	/* data */
-    0,		/* errno */
-    NULL,
-    NULL,
-    NULL,
-    local_which,
-
-    local_open,
-    local_close,
-    local_read,
-    local_write,
-    
-    local_opendir,
-    local_readdir,
-    local_closedir,
-    local_telldir,
-    local_seekdir,
-
-    local_stat,
-    local_lstat,
-    local_fstat,
-
-    local_chmod,
-    local_chown,
-    local_utime,
-
-    local_readlink,
-    local_symlink,
-    local_link,
-    local_unlink,
-
-    local_rename,
-    local_chdir,
-    local_errno,
-    local_lseek,
-    local_mknod,
-    
-    local_getid,
-    local_nothingisopen,
-    local_free,
-    
-    local_getlocalcopy,
-    local_ungetlocalcopy,
-    
-    local_mkdir,
-    local_rmdir,
-    
-    NULL,
-    NULL
+void
+init_localfs (void)
+{
+    vfs_local_ops.name = "localfs";
+    vfs_local_ops.flags = VFSF_LOCAL;
+    vfs_local_ops.which = local_which;
+    vfs_local_ops.open = local_open;
+    vfs_local_ops.close = local_close;
+    vfs_local_ops.read = local_read;
+    vfs_local_ops.write = local_write;
+    vfs_local_ops.opendir = local_opendir;
+    vfs_local_ops.readdir = local_readdir;
+    vfs_local_ops.closedir = local_closedir;
+    vfs_local_ops.stat = local_stat;
+    vfs_local_ops.lstat = local_lstat;
+    vfs_local_ops.fstat = local_fstat;
+    vfs_local_ops.chmod = local_chmod;
+    vfs_local_ops.chown = local_chown;
+    vfs_local_ops.utime = local_utime;
+    vfs_local_ops.readlink = local_readlink;
+    vfs_local_ops.symlink = local_symlink;
+    vfs_local_ops.link = local_link;
+    vfs_local_ops.unlink = local_unlink;
+    vfs_local_ops.rename = local_rename;
+    vfs_local_ops.chdir = local_chdir;
+    vfs_local_ops.ferrno = local_errno;
+    vfs_local_ops.lseek = local_lseek;
+    vfs_local_ops.mknod = local_mknod;
+    vfs_local_ops.getlocalcopy = local_getlocalcopy;
+    vfs_local_ops.ungetlocalcopy = local_ungetlocalcopy;
+    vfs_local_ops.mkdir = local_mkdir;
+    vfs_local_ops.rmdir = local_rmdir;
 #ifdef HAVE_MMAP
-    ,local_mmap,
-    local_munmap
+    vfs_local_ops.mmap = local_mmap;
+    vfs_local_ops.munmap = local_munmap;
 #endif
-};
-
-vfs vfs_nil_ops = {
-    NULL,	/* This is place of next pointer */
-    "nullfs",
-    0,		/* flags */
-    NULL,	/* prefix */
-    NULL,	/* data */
-    0,		/* errno */
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-
-    NULL,
-    NULL,
-    NULL,
-
-    NULL,
-    NULL,
-    NULL,
-
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    
-    local_getid,
-    local_nothingisopen,
-    local_free,
-    
-    NULL,
-    NULL,
-    
-    NULL,
-    NULL,
-    
-    NULL,
-    NULL
-#ifdef HAVE_MMAP
-    , NULL,
-    NULL
-#endif
-};
+    vfs_register_class (&vfs_local_ops);
+}
