@@ -16,7 +16,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 
@@ -37,13 +37,6 @@
 
 #ifdef HAVE_NETMASK_IFCONF
 
-#ifdef RELIANTUNIX
-/*
- * <unistd.h> must be included first to compile this file on ReliantUNIX
- */
-#include <unistd.h>
-#endif
-
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -241,6 +234,9 @@ this should cover most of the rest of systems
 #elif defined(HAVE_NETMASK_AIX)
 
 #include <stdio.h>
+#ifdef	HAVE_UNISTD_H
+#include <unistd.h>	/* close() declaration for gcc in fussy mode */
+#endif
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -253,9 +249,16 @@ this should cover most of the rest of systems
 #include <sys/sockio.h>
 #endif
 
+/*
+ * Prototype for gcc in fussy mode.
+ */
+
+int get_netmask(struct in_addr *ipaddr, struct in_addr *nmask);
+
 /****************************************************************************
 this one is for AIX
 ****************************************************************************/
+
  int get_netmask(struct in_addr *ipaddr, struct in_addr *nmask)
 {
 	char buff[2048];
