@@ -16,7 +16,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include "includes.h"
@@ -74,6 +74,7 @@ BOOL next_token(char **ptr,char *buff,char *sep, size_t bufsize)
   return(True);
 }
 
+#if 0
 /****************************************************************************
 Convert list of tokens to array; dependent on above routine.
 Uses last_ptr from above - bit of a hack.
@@ -110,7 +111,7 @@ char **toktocliplist(int *ctok, char *sep)
 
   return ret;
 }
-
+#endif /*0 */
 
 /*******************************************************************
   case insensitive string compararison
@@ -409,6 +410,7 @@ void strupper(char *s)
   }
 }
 
+#if 0
 /*******************************************************************
   convert a string to "normal" form
 ********************************************************************/
@@ -432,7 +434,7 @@ BOOL strisnormal(char *s)
 
   return(!strhasupper(s));
 }
-
+#endif /* 0 */
 
 /****************************************************************************
   string replace
@@ -580,7 +582,7 @@ BOOL trim_string(char *s,const char *front,const char *back)
   return(ret);
 }
 
-
+#if 0
 /****************************************************************************
 does a string have any uppercase chars in it?
 ****************************************************************************/
@@ -628,6 +630,7 @@ BOOL strhasupper(const char *s)
   return(False);
 }
 
+
 /****************************************************************************
 does a string have any lowercase chars in it?
 ****************************************************************************/
@@ -682,6 +685,7 @@ BOOL strhaslower(const char *s)
   }
   return(False);
 }
+#endif /*0 */
 
 /****************************************************************************
 find the number of chars in a string
@@ -832,7 +836,7 @@ char *StrnCpy(char *dest,const char *src,size_t n)
   return(dest);
 }
 
-
+#if 0
 /****************************************************************************
 like strncpy but copies up to the character marker.  always null terminates.
 returns a pointer to the character marker in the source string (src).
@@ -928,6 +932,7 @@ BOOL in_list(char *s,char *list,BOOL casesensitive)
   }
   return(False);
 }
+#endif /*0 */
 
 /* this is used to prevent lots of mallocs of size 1 */
 static char *null_string = NULL;
@@ -1032,7 +1037,7 @@ void string_sub(char *s,const char *pattern,const char *insert)
 	}
 }
 
-
+#if 0
 /****************************************************************************
 similar to string_sub() but allows for any character to be substituted. 
 Use with caution!
@@ -1058,6 +1063,7 @@ void all_string_sub(char *s,const char *pattern,const char *insert)
 	}
 }
 
+
 /****************************************************************************
  splits out the front and back at a separator.
 ****************************************************************************/
@@ -1089,3 +1095,4 @@ void split_at_last_component(char *path, char *front, char sep, char *back)
 		}
 	}
 }
+#endif /*0 */
