@@ -16,7 +16,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    
 */
 
@@ -26,8 +26,8 @@ extern pstring scope;
 extern int DEBUGLEVEL;
 
 /* nmbd.c sets this to True. */
-BOOL global_in_nmbd = False;
-
+const BOOL global_in_nmbd = False;
+#if 0
 /****************************************************************************
 interpret a node status response
 ****************************************************************************/
@@ -80,114 +80,7 @@ static void _interpret_node_status(char *p, char *master,char *rname)
   DEBUG(1,("num_good_sends=%d num_good_receives=%d\n",
 	       IVAL(p,20),IVAL(p,24)));
 }
-
-
-/****************************************************************************
-  do a netbios name status query on a host
-
-  the "master" parameter is a hack used for finding workgroups.
-  **************************************************************************/
-BOOL name_status(int fd,char *name,int name_type,BOOL recurse,
-		 struct in_addr to_ip,char *master,char *rname,
-		 void (*fn)(struct packet_struct *))
-{
-  BOOL found=False;
-  int retries = 2;
-  int retry_time = 5000;
-  struct timeval tval;
-  struct packet_struct p;
-  struct packet_struct *p2;
-  struct nmb_packet *nmb = &p.packet.nmb;
-  static int name_trn_id = 0;
-
-  memset((char *)&p,'\0',sizeof(p));
-
-  if (!name_trn_id) name_trn_id = ((unsigned)time(NULL)%(unsigned)0x7FFF) + 
-    ((unsigned)getpid()%(unsigned)100);
-  name_trn_id = (name_trn_id+1) % (unsigned)0x7FFF;
-
-  nmb->header.name_trn_id = name_trn_id;
-  nmb->header.opcode = 0;
-  nmb->header.response = False;
-  nmb->header.nm_flags.bcast = False;
-  nmb->header.nm_flags.recursion_available = False;
-  nmb->header.nm_flags.recursion_desired = False;
-  nmb->header.nm_flags.trunc = False;
-  nmb->header.nm_flags.authoritative = False;
-  nmb->header.rcode = 0;
-  nmb->header.qdcount = 1;
-  nmb->header.ancount = 0;
-  nmb->header.nscount = 0;
-  nmb->header.arcount = 0;
-
-  make_nmb_name(&nmb->question.question_name,name,name_type,scope);
-
-  nmb->question.question_type = 0x21;
-  nmb->question.question_class = 0x1;
-
-  p.ip = to_ip;
-  p.port = NMB_PORT;
-  p.fd = fd;
-  p.timestamp = time(NULL);
-  p.packet_type = NMB_PACKET;
-
-  GetTimeOfDay(&tval);
-
-  if (!send_packet(&p)) 
-    return(False);
-
-  retries--;
-
-  while (1)
-    {
-      struct timeval tval2;
-      GetTimeOfDay(&tval2);
-      if (TvalDiff(&tval,&tval2) > retry_time) {
-	if (!retries) break;
-	if (!found && !send_packet(&p))
-	  return False;
-	GetTimeOfDay(&tval);
-	retries--;
-      }
-
-      if ((p2=receive_packet(fd,NMB_PACKET,90)))
-	{     
-	  struct nmb_packet *nmb2 = &p2->packet.nmb;
-      debug_nmb_packet(p2);
-
-	  if (nmb->header.name_trn_id != nmb2->header.name_trn_id ||
-	      !nmb2->header.response) {
-	    /* its not for us - maybe deal with it later */
-	    if (fn) 
-	      fn(p2);
-	    else
-	      free_packet(p2);
-	    continue;
-	  }
-	  
-	  if (nmb2->header.opcode != 0 ||
-	      nmb2->header.nm_flags.bcast ||
-	      nmb2->header.rcode ||
-	      !nmb2->header.ancount ||
-	      nmb2->answers->rr_type != 0x21) {
-	    /* XXXX what do we do with this? could be a redirect, but
-	       we'll discard it for the moment */
-	    free_packet(p2);
-	    continue;
-	  }
-
-	  _interpret_node_status(&nmb2->answers->rdata[0], master,rname);
-	  free_packet(p2);
-	  return(True);
-	}
-    }
-  
-
-  DEBUG(0,("No status response (this is not unusual)\n"));
-
-  return(False);
-}
-
+#endif /* 0 */
 
 /****************************************************************************
   do a netbios name query to find someones IP
@@ -228,7 +121,7 @@ struct in_addr *name_query(int fd,const char *name,int name_type, BOOL bcast,BOO
   nmb->header.nscount = 0;
   nmb->header.arcount = 0;
 
-  make_nmb_name(&nmb->question.question_name,name,name_type,scope);
+  make_nmb_name(&nmb->question.question_name,name,name_type);
 
   nmb->question.question_type = 0x20;
   nmb->question.question_class = 0x1;
@@ -334,7 +227,7 @@ FILE *startlmhosts(char *fname)
 {
   FILE *fp = sys_fopen(fname,"r");
   if (!fp) {
-    DEBUG(4,("startlmhosts: Can't open lmhosts file %s. Error was %s\n",
+    DEBUG(4,("startlmhosts: Cannot open lmhosts file %s. Error was %s\n",
              fname, strerror(errno)));
     return NULL;
   }
@@ -646,7 +539,7 @@ BOOL resolve_name(const char *name, struct in_addr *return_ip, int name_type)
 }
 
 
-
+#if 0
 /********************************************************
 find the IP address of the master browser or DMB for a workgroup
 *********************************************************/
@@ -656,3 +549,4 @@ BOOL find_master_ip(char *group, struct in_addr *master_ip)
 
 	return resolve_name(group, master_ip, 0x1B);
 }
+#endif	/* 0 */
